import java.awt.*;
import java.awt.Image;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.jfree.chart.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.labels.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.xy.*;
import org.jfree.data.xy.*;
import org.jfree.ui.TextAnchor;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import jxl.*;
import jxl.read.biff.BiffException;
import jxl.write.DateFormat;
import jxl.write.DateTime;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.WritableCellFormat;
import jxl.write.WritableHyperlink;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import jxl.write.biff.RowsExceededException;

public class GUI {

	public static LinkedList<GeneticAlgorithm<?>> gaList = new LinkedList<GeneticAlgorithm<?>>();

	public static String fs = System.getProperty("file.separator");

	private static final int screenDevice = 0;

	private final int controlHeight = 220;

	private final int graphWidthOffset = 200;

	private final int graphHeigtOffset = 120;

	private final int graphWidth = 800;

	private int graphHeight;

	private GraphicsEnvironment ge;

	private GraphicsDevice[] gs;

	private GraphicsConfiguration gc;

	private Dimension screenSize;

	private boolean running;

	private GeneticAlgorithm<?> ga;

	private Display display;

	private ExcelHandler excelHandler;

	private JScrollPane scrollPane;

	private JFrame frame;

	private JMenuBar menuBar;

	private JMenu menu;

	private JMenuItem menuItem;

	private JPanel controlPanel, objectiveTypePanel, encodingTypePanel,
			functionPanel, combinatorialPanel, singlePanel, multiPanel,
			variableRangePanel, parameterPanel, sequencePanel;

	private JRadioButton singleRB, multiRB, bitStringRB, realCodedRB;

	private JButton startButton, pauseButton, stepButton;

	private JCheckBox testSelectivePressureCheckBox;

	private JTextField chromosomeSizeTF, populationSizeTF, generationSizeTF,
			functionS1TF, functionM1TF, functionM2TF, xLowerTF, xUpperTF;

	private JLabel chromosomeSizeL, populationSizeL, generationSizeL,
			function1L, function2L, xL;

	private JList combinatorialFunctionList, singleFunctionList,
			multiFunctionList;

	private JComboBox viewTypeCB;

	private int componentMaxHeight = 150,
			subPanelHeight = componentMaxHeight + 40;

	private Dimension objectivePanelDimension = new Dimension(200,
			subPanelHeight / 3), functionSTFDimension = new Dimension(200, 25),
			functionMTFDimension = new Dimension(200, 25),
			domainTFDimension = new Dimension(70, 20),
			subFunctionPanelDimension = new Dimension(300, subPanelHeight),
			realValueFunctionListDimension = new Dimension(70,
					componentMaxHeight);

	private void initialize() {
		ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		gs = ge.getScreenDevices();
		gc = gs[screenDevice].getDefaultConfiguration();
		screenSize = new Dimension(gc.getBounds().width, gc.getBounds().height);
		graphHeight = Math.min(900, screenSize.height - 50) - controlHeight
				- graphHeigtOffset;
		frame = new JFrame("Evolutionary Algorithm", gc);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		excelHandler = new ExcelHandler();
		display = new Display(graphWidth, graphHeight, excelHandler);
		scrollPane = new JScrollPane(display);
		scrollPane.setPreferredSize(new Dimension(
				graphWidth + graphWidthOffset, graphHeight + graphHeigtOffset));

		functionPanel = new JPanel();
		functionPanel.setOpaque(true);
		functionPanel.setLayout(new CardLayout());

		bitStringRB = new JRadioButton("Bit-String");
		bitStringRB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent arg0) {
				CardLayout c1 = (CardLayout) functionPanel.getLayout();
				c1.show(functionPanel, "COMBINATORIAL");
				variableRangePanel.setVisible(false);
				setDefaultParameter();
			}
		});
		realCodedRB = new JRadioButton("Real-Coded");
		realCodedRB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent arg0) {
				CardLayout c1 = (CardLayout) functionPanel.getLayout();
				c1.show(functionPanel, "SINGLE");
				variableRangePanel.setVisible(true);
				setDefaultParameter();
			}
		});

		ButtonGroup encodingGroup = new ButtonGroup();
		encodingGroup.add(bitStringRB);
		encodingGroup.add(realCodedRB);

		singleRB = new JRadioButton("Single");
		singleRB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent arg0) {
				bitStringRB.setEnabled(true);
				realCodedRB.setEnabled(true);
				CardLayout c1 = (CardLayout) functionPanel.getLayout();
				if (bitStringRB.isSelected()) {
					c1.show(functionPanel, "COMBINATORIAL");
					variableRangePanel.setVisible(false);
				} else if (realCodedRB.isSelected()) {
					c1.show(functionPanel, "SINGLE");
					variableRangePanel.setVisible(true);
				}
				setDefaultParameter();
			}
		});

		multiRB = new JRadioButton("Multiple");
		multiRB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent arg0) {
				realCodedRB.setSelected(true);
				bitStringRB.setEnabled(false);
				realCodedRB.setEnabled(false);
				CardLayout c1 = (CardLayout) functionPanel.getLayout();
				c1.show(functionPanel, "MULTIPLE");
				variableRangePanel.setVisible(true);
				setDefaultParameter();
			}
		});

		ButtonGroup objectiveGroup = new ButtonGroup();
		objectiveGroup.add(singleRB);
		objectiveGroup.add(multiRB);

		parameterPanel = new JPanel();
		parameterPanel.setLayout(new GridLayout(0, 2));
		parameterPanel.setBorder(BorderFactory.createTitledBorder("Parameter"));
		chromosomeSizeL = new JLabel("Chromosome: ");
		chromosomeSizeL.setHorizontalAlignment(JLabel.RIGHT);
		populationSizeL = new JLabel("Population: ");
		populationSizeL.setHorizontalAlignment(JLabel.RIGHT);
		generationSizeL = new JLabel("Generation: ");
		generationSizeL.setHorizontalAlignment(JLabel.RIGHT);
		chromosomeSizeTF = new JTextField();
		populationSizeTF = new JTextField();
		generationSizeTF = new JTextField();

		parameterPanel.add(chromosomeSizeL);
		parameterPanel.add(chromosomeSizeTF);
		parameterPanel.add(populationSizeL);
		parameterPanel.add(populationSizeTF);
		parameterPanel.add(generationSizeL);
		parameterPanel.add(generationSizeTF);

		// Combinatorial Panel
		combinatorialPanel = new JPanel();
		combinatorialPanel.setBorder(BorderFactory
				.createTitledBorder("Single Objective"));
		combinatorialPanel.setPreferredSize(subFunctionPanelDimension);

		DefaultListModel combinatorialFunctionListModel = new DefaultListModel();
		combinatorialFunctionListModel.add(0, "Valley");
		combinatorialFunctionListModel.add(1, "ONE-MAX");
		combinatorialFunctionListModel.add(2, "Subsetsum");
		combinatorialFunctionListModel.add(3, "MMDP");
		combinatorialFunctionListModel.add(4, "P-PEAKS");
		combinatorialFunctionListModel.add(5, "COUNTSAT");
		combinatorialFunctionListModel.add(6, "QAP");
		combinatorialFunctionList = new JList(combinatorialFunctionListModel);
		combinatorialFunctionList
				.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		combinatorialFunctionList.setSelectedIndex(0);
		combinatorialFunctionList
				.addListSelectionListener(new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent arg0) {
						setDefaultParameter();
					}
				});
		JScrollPane combinatorialFunctionListScroller = new JScrollPane(
				combinatorialFunctionList);
		combinatorialFunctionListScroller.setPreferredSize(new Dimension(170,
				componentMaxHeight - 20));

		testSelectivePressureCheckBox = new JCheckBox("Test Selective Pressure");
		testSelectivePressureCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setDefaultParameter();
			}
		});

		combinatorialPanel.add(combinatorialFunctionListScroller);
		combinatorialPanel.add(testSelectivePressureCheckBox);

		// Single Objective Function Panel
		singlePanel = new JPanel(new GridBagLayout());
		singlePanel.setBorder(BorderFactory
				.createTitledBorder("Single Objective"));
		singlePanel.setPreferredSize(subFunctionPanelDimension);

		DefaultListModel singleFunctionListModel = new DefaultListModel();
		for (int i = 0; i < SingleObjectiveTestFunction.FUNCTION_LIST_STRING.length; i++)
			singleFunctionListModel.add(i, "Test" + (i + 1));
		singleFunctionList = new JList(singleFunctionListModel);
		singleFunctionList
				.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		singleFunctionList.setSelectedIndex(0);
		singleFunctionList
				.addListSelectionListener(new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent arg0) {
						int index = singleFunctionList.getSelectedIndex();
						functionS1TF
								.setText(SingleObjectiveTestFunction.FUNCTION_LIST_STRING[index]);
						setDefaultParameter();
					}
				});
		JScrollPane singleFunctionListScroller = new JScrollPane(
				singleFunctionList);
		singleFunctionListScroller
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		singleFunctionListScroller
				.setPreferredSize(realValueFunctionListDimension);

		function1L = new JLabel("Function 1");
		functionS1TF = new JTextField();
		functionS1TF.setPreferredSize(functionSTFDimension);
		functionS1TF.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		int index = singleFunctionList.getSelectedIndex();
		functionS1TF
				.setText(SingleObjectiveTestFunction.FUNCTION_LIST_STRING[index]);

		functionS1TF.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
			}

			public void insertUpdate(DocumentEvent e) {
				functionS1TF.setForeground(Color.BLACK);
			}

			public void removeUpdate(DocumentEvent e) {
				functionS1TF.setForeground(Color.BLACK);
			}
		});

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridheight = 4;
		c.insets = new Insets(0, 0, 0, 10);
		singlePanel.add(singleFunctionListScroller, c);
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(0, 0, 0, 0);
		c.gridheight = 1;
		c.gridx++;
		singlePanel.add(function1L, c);
		c.gridy++;
		c.insets = new Insets(0, 0, 10, 0);
		singlePanel.add(functionS1TF, c);

		// Multiple Objective Function Panel
		multiPanel = new JPanel(new GridBagLayout());
		multiPanel.setBorder(BorderFactory
				.createTitledBorder("Multiple Objective"));
		multiPanel.setPreferredSize(subFunctionPanelDimension);

		DefaultListModel multiFunctionListModel = new DefaultListModel();
		for (int i = 0; i < MultiObjectiveTestFunction.FUNCTION_LIST_STRING.length; i++)
			multiFunctionListModel.add(i, "Test" + (i + 1));
		multiFunctionList = new JList(multiFunctionListModel);
		multiFunctionList
				.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		multiFunctionList.setSelectedIndex(0);
		multiFunctionList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent arg0) {
				int index = multiFunctionList.getSelectedIndex();
				functionM1TF
						.setText(MultiObjectiveTestFunction.FUNCTION_LIST_STRING[index][0]);
				functionM2TF
						.setText(MultiObjectiveTestFunction.FUNCTION_LIST_STRING[index][1]);
				setDefaultParameter();
			}
		});
		JScrollPane multiFunctionListScroller = new JScrollPane(
				multiFunctionList);
		multiFunctionListScroller
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		multiFunctionListScroller
				.setPreferredSize(realValueFunctionListDimension);

		index = multiFunctionList.getSelectedIndex();
		function1L = new JLabel("Function 1");
		function2L = new JLabel("Function 2");
		functionM1TF = new JTextField();
		functionM1TF.setPreferredSize(functionMTFDimension);
		functionM1TF.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		functionM1TF
				.setText(MultiObjectiveTestFunction.FUNCTION_LIST_STRING[index][0]);
		functionM1TF.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
			}

			public void insertUpdate(DocumentEvent e) {
				functionM1TF.setForeground(Color.BLACK);
			}

			public void removeUpdate(DocumentEvent e) {
				functionM1TF.setForeground(Color.BLACK);
			}
		});
		functionM2TF = new JTextField();
		functionM2TF.setPreferredSize(functionMTFDimension);
		functionM2TF.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		functionM2TF
				.setText(MultiObjectiveTestFunction.FUNCTION_LIST_STRING[index][1]);
		functionM2TF.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
			}

			public void insertUpdate(DocumentEvent e) {
				functionM2TF.setForeground(Color.BLACK);
			}

			public void removeUpdate(DocumentEvent e) {
				functionM2TF.setForeground(Color.BLACK);
			}
		});

		variableRangePanel = new JPanel(new FlowLayout());
		variableRangePanel.setBorder(BorderFactory
				.createTitledBorder("Variable Range"));
		xL = new JLabel(" < x < ");
		xLowerTF = new JTextField(
				MultiObjectiveTestFunction.VARIABLE_DOMAIN[index][0] + "");
		xLowerTF.setPreferredSize(domainTFDimension);
		xLowerTF.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
			}

			public void insertUpdate(DocumentEvent e) {
				xLowerTF.setForeground(Color.BLACK);
			}

			public void removeUpdate(DocumentEvent e) {
				xLowerTF.setForeground(Color.BLACK);
			}
		});
		xUpperTF = new JTextField(
				MultiObjectiveTestFunction.VARIABLE_DOMAIN[index][1] + "");
		xUpperTF.setPreferredSize(domainTFDimension);
		xUpperTF.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
			}

			public void insertUpdate(DocumentEvent e) {
				xUpperTF.setForeground(Color.BLACK);
			}

			public void removeUpdate(DocumentEvent e) {
				xUpperTF.setForeground(Color.BLACK);
			}
		});
		variableRangePanel.add(xLowerTF);
		variableRangePanel.add(xL);
		variableRangePanel.add(xUpperTF);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridheight = 4;
		c.insets = new Insets(0, 0, 0, 10);
		multiPanel.add(multiFunctionListScroller, c);
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(0, 0, 0, 0);
		c.gridheight = 1;
		c.gridx++;
		multiPanel.add(function1L, c);
		c.gridy++;
		c.insets = new Insets(0, 0, 10, 0);
		multiPanel.add(functionM1TF, c);
		c.gridy++;
		c.insets = new Insets(0, 0, 0, 0);
		multiPanel.add(function2L, c);
		c.gridy++;
		multiPanel.add(functionM2TF, c);

		functionPanel.add(combinatorialPanel, "COMBINATORIAL");
		functionPanel.add(singlePanel, "SINGLE");
		functionPanel.add(multiPanel, "MULTIPLE");

		objectiveTypePanel = new JPanel();
		objectiveTypePanel.setPreferredSize(objectivePanelDimension);
		objectiveTypePanel.setBorder(BorderFactory
				.createTitledBorder("Objective"));
		objectiveTypePanel.add(singleRB);
		objectiveTypePanel.add(multiRB);

		encodingTypePanel = new JPanel();
		encodingTypePanel.setBorder(BorderFactory
				.createTitledBorder("Encoding"));
		encodingTypePanel.add(bitStringRB);
		encodingTypePanel.add(realCodedRB);

		startButton = new JButton("Start");
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (running) {
					ga.STEP = false;
					startButton.setText("Start");
					ga.stop();
					running = false;
					enablePanel();
				} else {
					try {
						setGeneticAlgorithm();
						ga.setParameter(Integer.parseInt(chromosomeSizeTF
								.getText()), Integer.parseInt(generationSizeTF
								.getText()), Integer.parseInt(populationSizeTF
								.getText()));
					} catch (Exception e) {
						System.err.println(e);
						return;
					}
					startButton.setText("Stop");
					ga.start();
					running = true;
					disablePanel();
				}
				if (ga.STEP)
					pauseButton.setText("|>");
				else {
					pauseButton.setText("||");
				}
			}
		});

		// default start value
		// realCodedRB.setSelected(true);
		// multiRB.setSelected(true);
		bitStringRB.setSelected(true);
		singleRB.setSelected(true);

		pauseButton = new JButton("||");
		pauseButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!ga.STEP)
					pauseButton.setText("|>");
				else {
					ga.stepNum++;
					pauseButton.setText("||");
				}
				ga.STEP = !ga.STEP;
				synchronized (ga.t) {
					ga.t.notify();
				}
			}
		});
		stepButton = new JButton(">>");
		stepButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ga.stepNum++;
				synchronized (ga.t) {
					ga.t.notify();
				}
			}
		});
		sequencePanel = new JPanel();
		sequencePanel.setBorder(BorderFactory.createTitledBorder("Sequence"));
		sequencePanel.add(pauseButton);
		sequencePanel.add(stepButton);

		String[] viewTypeList = { "Graph", "Selection Probability",
				"Local Diversity" };
		viewTypeCB = new JComboBox(viewTypeList);
		viewTypeCB.setEnabled(false);
		viewTypeCB.setBorder(BorderFactory.createTitledBorder("View Type"));
		viewTypeCB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CardLayout c = (CardLayout) display.getLayout();
				if (viewTypeCB.getSelectedItem().equals("Graph"))
					if (testSelectivePressureCheckBox.isSelected())
						c.show(display, Display.TEST_SELECTIVE_PRESSURE);
					else if (singleRB.isSelected()
							&& bitStringRB.isSelected()
							&& combinatorialFunctionList.getSelectedIndex() == 0)
						c.show(display, Display.GRID);
					else
						c.show(display, Display.GRAPH);
				else if (viewTypeCB.getSelectedItem().equals(
						"Selection Probability"))
					c.show(display, Display.SELECTION_PROBABILITY);
				else if (viewTypeCB.getSelectedItem().equals("Local Diversity"))
					c.show(display, Display.LOCAL_DIVERSITY);
			}
		});

		controlPanel = new JPanel(new GridBagLayout());
		controlPanel.setPreferredSize(new Dimension(graphWidth
				+ graphWidthOffset / 2, controlHeight));
		controlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.WHITE), "Setting", TitledBorder.CENTER,
				TitledBorder.TOP));
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 0;
		controlPanel.add(objectiveTypePanel, c);
		c.gridy++;
		controlPanel.add(encodingTypePanel, c);
		c.gridy++;
		controlPanel.add(variableRangePanel, c);
		c.gridx++;
		c.gridy = 0;
		c.gridheight = 3;
		controlPanel.add(parameterPanel, c);
		c.gridx++;
		controlPanel.add(functionPanel, c);
		c.gridx++;
		c.gridheight = 1;
		controlPanel.add(startButton, c);
		c.gridy++;
		controlPanel.add(sequencePanel, c);
		c.gridy++;
		controlPanel.add(viewTypeCB, c);

		menuBar = new JMenuBar();
		menu = new JMenu("Options");
		menuBar.add(menu);
		menuItem = new JMenuItem("Parameter...");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new ParameterFrame();
			}
		});
		menu.add(menuItem);

		frame.setJMenuBar(menuBar);
		frame.setPreferredSize(new Dimension(graphWidth + graphWidthOffset,
				graphHeight + graphHeigtOffset + controlHeight));
		// frame.setResizable(false);
		frame.setLayout(new BorderLayout());
		frame.add(scrollPane, BorderLayout.CENTER);
		frame.add(controlPanel, BorderLayout.PAGE_END);
		frame.pack();
		frame.setVisible(true);
	}

	public void setGeneticAlgorithm() throws Exception {
		Expression[] expr = new Expression[2];
		CardLayout c = (CardLayout) display.getLayout();
		if (testSelectivePressureCheckBox.isSelected()) {
			ga = new TestSelectivePressure(display);
			ga.STEP = GeneticAlgorithm.STEP_ALGORITHM;
			c.show(display, Display.TEST_SELECTIVE_PRESSURE);
			return;
		}
		c.show(display, Display.GRAPH);
		if (singleRB.isSelected()) {
			if (bitStringRB.isSelected()) {
				if (combinatorialFunctionList.getSelectedIndex() == 0) {
					ga = new GridGeneticAlgorithm(display);
					c.show(display, Display.GRID);
				} else if (combinatorialFunctionList.getSelectedIndex() == 6) {
					ga = new PermutationGeneticAlgorithm(display, 6);
					chromosomeSizeTF.setEnabled(false);
				} else {
					ga = new BitStringGeneticAlgorithm(display,
							combinatorialFunctionList.getSelectedIndex());
				}
			} else if (realCodedRB.isSelected()) {
				if (GeneticAlgorithm.USE_PREDEFINED_FUNCTION)
					ga = new RealCodedGeneticAlgorithm(display,
							singleFunctionList.getSelectedIndex());
				else {
					Exception exception = null;
					try {
						expr[0] = new Expression(functionS1TF.getText());
					} catch (SyntaxException e) {
						functionS1TF.setForeground(Color.RED);
						exception = e;
					}
					try {
						SingleObjectiveTestFunction.USER_DEFINED_VARIABLE_DOMAIN[0] = Double
								.parseDouble(xLowerTF.getText());
					} catch (NumberFormatException e) {
						xLowerTF.setForeground(Color.RED);
						exception = e;
					}
					try {
						SingleObjectiveTestFunction.USER_DEFINED_VARIABLE_DOMAIN[1] = Double
								.parseDouble(xUpperTF.getText());
					} catch (NumberFormatException e) {
						xUpperTF.setForeground(Color.RED);
						exception = e;
					}
					if (exception != null)
						throw exception;
					ga = new RealCodedGeneticAlgorithm(display, expr);
				}
			}
		} else if (multiRB.isSelected()) {
			if (GeneticAlgorithm.USE_PREDEFINED_FUNCTION)
				ga = new MultiObjectiveRealCodedGeneticAlgorithm(display,
						multiFunctionList.getSelectedIndex());
			else {
				Exception exception = null;
				try {
					expr[0] = new Expression(functionM1TF.getText());
					MultiObjectiveTestFunction.FUNCTION_LIST_STRING[multiFunctionList
							.getSelectedIndex()][0] = functionM1TF.getText();
				} catch (SyntaxException e) {
					functionM1TF.setForeground(Color.RED);
					exception = e;
				}
				try {
					expr[1] = new Expression(functionM2TF.getText());
					MultiObjectiveTestFunction.FUNCTION_LIST_STRING[multiFunctionList
							.getSelectedIndex()][1] = functionM2TF.getText();
				} catch (SyntaxException e) {
					functionM2TF.setForeground(Color.RED);
					exception = e;
				}
				try {
					MultiObjectiveTestFunction.USER_DEFINED_VARIABLE_DOMAIN[0] = Double
							.parseDouble(xLowerTF.getText());
				} catch (NumberFormatException e) {
					xLowerTF.setForeground(Color.RED);
					exception = e;
				}
				try {
					MultiObjectiveTestFunction.USER_DEFINED_VARIABLE_DOMAIN[1] = Double
							.parseDouble(xUpperTF.getText());
				} catch (NumberFormatException e) {
					xUpperTF.setForeground(Color.RED);
					exception = e;
				}
				if (exception != null)
					throw exception;
				ga = new MultiObjectiveRealCodedGeneticAlgorithm(display, expr);
			}
			display.setOptimizer((MultiObjectiveRealCodedGeneticAlgorithm) ga);
		} else {
			ga = new BitStringGeneticAlgorithm(display, 0);
		}
		ga.STEP = GeneticAlgorithm.STEP_ALGORITHM;
		display.isDisplayDiversity = GeneticAlgorithm.SHOW_DIVERSITY;
	}

	public void setDefaultParameter() {
		chromosomeSizeTF.setEnabled(true);
		setVariableRange();
		try {
			setGeneticAlgorithm();
		} catch (Exception e) {
			System.err.println(e);
			return;
		}
		chromosomeSizeTF.setText(ga.CHROMOSOME_SIZE + "");
		populationSizeTF.setText(ga.POPULATION_SIZE + "");
		generationSizeTF.setText(ga.GENERATION_SIZE + "");
		display.updateAlgorithmStatus(null, null, null);
	}

	public void setVariableRange() {
		if (singleRB.isSelected()) {
			int index = singleFunctionList.getSelectedIndex();
			xLowerTF.setText(""
					+ SingleObjectiveTestFunction.VARIABLE_DOMAIN[index][0]);
			xUpperTF.setText(""
					+ SingleObjectiveTestFunction.VARIABLE_DOMAIN[index][1]);
		} else if (multiRB.isSelected()) {
			int index = multiFunctionList.getSelectedIndex();
			xLowerTF.setText(""
					+ MultiObjectiveTestFunction.VARIABLE_DOMAIN[index][0]);
			xUpperTF.setText(""
					+ MultiObjectiveTestFunction.VARIABLE_DOMAIN[index][1]);
		}
	}

	public void enablePanel() {
		singleRB.setEnabled(true);
		multiRB.setEnabled(true);
		if (!multiRB.isSelected()) {
			bitStringRB.setEnabled(true);
			realCodedRB.setEnabled(true);
		}
		chromosomeSizeTF.setEnabled(true);
		populationSizeTF.setEnabled(true);
		generationSizeTF.setEnabled(true);
		combinatorialFunctionList.setEnabled(true);
		singleFunctionList.setEnabled(true);
		multiFunctionList.setEnabled(true);
		functionS1TF.setEnabled(true);
		functionM1TF.setEnabled(true);
		functionM2TF.setEnabled(true);
		xLowerTF.setEnabled(true);
		xUpperTF.setEnabled(true);
		viewTypeCB.setSelectedIndex(0);
		viewTypeCB.setEnabled(false);
		testSelectivePressureCheckBox.setEnabled(true);
		if (testSelectivePressureCheckBox.isSelected())
			viewTypeCB.addItem("Local Diversity");
	}

	public void disablePanel() {
		singleRB.setEnabled(false);
		multiRB.setEnabled(false);
		bitStringRB.setEnabled(false);
		realCodedRB.setEnabled(false);
		chromosomeSizeTF.setEnabled(false);
		populationSizeTF.setEnabled(false);
		generationSizeTF.setEnabled(false);
		combinatorialFunctionList.setEnabled(false);
		singleFunctionList.setEnabled(false);
		multiFunctionList.setEnabled(false);
		functionS1TF.setEnabled(false);
		functionM1TF.setEnabled(false);
		functionM2TF.setEnabled(false);
		xLowerTF.setEnabled(false);
		xUpperTF.setEnabled(false);
		if (ga.STRUCTURE == GeneticAlgorithm.CELLULAR2D)
			viewTypeCB.setEnabled(true);
		testSelectivePressureCheckBox.setEnabled(false);
		if (testSelectivePressureCheckBox.isSelected())
			viewTypeCB.removeItem("Local Diversity");
	}

	public static void printEstimatedTimeToFinish(int i, int j, int n,
			int repeat, long averageTime, double[][] probabilityParam) {
		System.out.println("Experiment No." + i + " " + j + " " + n);
		if (n > 0) {
			System.out.println("Estimated time to finish this parameter = "
					+ ((repeat - n) * averageTime) / 60000 + " mins");
			Date d = new Date();
			d.setTime(d.getTime() + (repeat - n) * averageTime);
			System.out.println("Estimated finish time of this parameter = "
					+ d.toString());
			int parameterLeft = 0;
			for (int k = i + 1; k < probabilityParam.length; k++)
				parameterLeft += probabilityParam[k].length;
			parameterLeft += probabilityParam[i].length - j - 1;
			System.out.println("There are " + parameterLeft
					+ " parameters left");
			System.out.println("Estimated time to finish = "
					+ ((repeat - n + repeat * parameterLeft) * averageTime)
					/ 60000 + " mins");
			d.setTime(d.getTime() + (repeat * parameterLeft * averageTime));
			System.out.println("Estimated finish time = " + d.toString());
		}
	}

	public void startBackgroundExperiment(String neighborHoodSizeString,
			String probabilityParamString, int repeat) {
		long startTime = 0, totalTime = 0, averageTime = 0;
		int[] neighborHoodSize = { 1, 20 };
		double[][] probabilityParam = { { 0, 2, 4 }, { 10, 20, 30 } };
		boolean useDefaultParameter = false;
		if (!useDefaultParameter) {
			String[] neighborHoodSizeResult = neighborHoodSizeString.split(",");
			neighborHoodSize = new int[neighborHoodSizeResult.length];
			for (int i = 0; i < neighborHoodSizeResult.length; i++)
				neighborHoodSize[i] = Integer
						.parseInt(neighborHoodSizeResult[i]);
			String[] probabilityParamResult = probabilityParamString.split(",");
			probabilityParam = new double[neighborHoodSizeResult.length][probabilityParamResult.length];
			for (int i = 0; i < neighborHoodSizeResult.length; i++)
				for (int j = 0; j < probabilityParamResult.length; j++)
					probabilityParam[i][j] = Double
							.parseDouble(probabilityParamResult[j]);
		}
		GeneticAlgorithm.PRINT_GRAPH = false;

		for (int i = 0; i < probabilityParam.length; i++) {
			for (int j = 0; j < probabilityParam[i].length; j++) {
				totalTime = 0;
				for (int n = 0; n < repeat; n++) {
					printEstimatedTimeToFinish(i, j, n, repeat, averageTime,
							probabilityParam);
					GeneticAlgorithm<?> ga;
					startTime = System.currentTimeMillis();
					// ga = new TestSelectivePressure(new Display(0, 0,
					// new ExcelHandler()));
					// ga = new PermutationGeneticAlgorithm(new Display(0, 0,
					// new ExcelHandler()), 6);
					// ga = new GridGeneticAlgorithm(new Display(0, 0,
					// new ExcelHandler()));
					ga = new BitStringGeneticAlgorithm(new Display(0, 0,
							new ExcelHandler()), 3);
					// ga.POLYNOMIAL_PARAMETER = BitStringTestFunction
					// .convertParamC2P(probabilityParam[i][j]);
					if (ga.SELECTION_SCHEME == GeneticAlgorithm.POLYNOMIAL_SELECTION) {
						ga.POLYNOMIAL_PARAMETER = probabilityParam[i][j];
						System.out.println(ga.POLYNOMIAL_PARAMETER);
					} else if (ga.SELECTION_SCHEME == GeneticAlgorithm.CENTRIC_SELECTION) {
						ga.CENTRIC_PARAMETER = probabilityParam[i][j];
						System.out.println(ga.CENTRIC_PARAMETER);
					}
					ga.neighborhoodSize = neighborHoodSize[i];
					ga.setParameter(ga.chromosomeSize, ga.generationSize,
							ga.populationSize);
					ga.STEP = false;
					ga.start();
					gaList.addFirst(ga);
					synchronized (gaList) {
						while (gaList.size() == 1)
							try {
								gaList.wait();
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
					}
					totalTime += (System.currentTimeMillis() - startTime);
					averageTime = totalTime / (n + 1);
				}
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public static void main(String[] args) {
		GUI gui = new GUI();
		if (args.length >= 3) {
			if (args.length >= 4) {
				int repeat = 1;
				if (args.length == 5)
					repeat = Integer.parseInt(args[4])
							- Integer.parseInt(args[3]) + 1;
				for (int i = 0; i < repeat; i++) {
					if (args.length == 5)
						ExcelHandler.EXCEL_OUTPUT_SUFFIX = (Integer
								.parseInt(args[3]) + i)
								+ "";
					else
						ExcelHandler.EXCEL_OUTPUT_SUFFIX = args[3];
					ExcelHandler.updatePath();
					gui.startBackgroundExperiment(args[0], args[1], Integer
							.parseInt(args[2]));
				}
			} else
				gui.startBackgroundExperiment(args[0], args[1], Integer
						.parseInt(args[2]));
		} else {
			// gui.startBackgroundExperiment("1", "0.0", 1);
			gui.initialize();
		}
	}
}

class ParameterFrame {
	public JFrame frame;

	public JPanel booleanPanel, doublePanel, buttonPanel;

	public ArrayList<JCheckBox> checkBoxes;

	public ArrayList<JTextField> textFields;

	public JButton ok, cancel;

	public ParameterFrame() {
		booleanPanel = new JPanel();
		booleanPanel
				.setLayout(new BoxLayout(booleanPanel, BoxLayout.PAGE_AXIS));
		booleanPanel.setBorder(BorderFactory
				.createTitledBorder("Boolean Parameters"));
		readBooleanParameter();

		doublePanel = new JPanel();
		doublePanel.setLayout(new GridLayout(0, 2));
		doublePanel.setBorder(BorderFactory
				.createTitledBorder("Algorithm Parameters"));
		readDoubleParameter();

		ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				writeBooleanParameter();
				writeDoubleParameter();
				frame.dispose();
			}
		});
		cancel = new JButton("Cancel");
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				frame.dispose();
			}
		});
		buttonPanel = new JPanel();
		buttonPanel.add(ok);
		buttonPanel.add(cancel);
		frame = new JFrame("Parameter");
		frame.setLayout(new FlowLayout());
		frame.add(booleanPanel);
		frame.add(doublePanel);
		frame.add(buttonPanel);
		frame.setPreferredSize(new Dimension(350, 600));
		frame.pack();
		frame.setVisible(true);
	}

	public void readBooleanParameter() {
		GeneticAlgorithm.initBooleanParameter();
		Set<String> keys = GeneticAlgorithm.booleanParameter.keySet();
		checkBoxes = new ArrayList<JCheckBox>();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			Boolean b = GeneticAlgorithm.booleanParameter.get(key);
			checkBoxes.add(new JCheckBox(key, b));
			this.booleanPanel.add(checkBoxes.get(checkBoxes.size() - 1));
		}
	}

	public void writeBooleanParameter() {
		for (JCheckBox checkBox : checkBoxes) {
			GeneticAlgorithm.booleanParameter.put(checkBox.getText(),
					new Boolean(checkBox.isSelected()));
		}
		GeneticAlgorithm.setBooleanParameter();
	}

	public void readDoubleParameter() {
		GeneticAlgorithm.initDoubleParameter();
		Set<String> keys = GeneticAlgorithm.doubleParameter.keySet();
		textFields = new ArrayList<JTextField>();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			Double d = GeneticAlgorithm.doubleParameter.get(key);
			JTextField temp = new JTextField();
			temp.setText("" + d);
			temp.setName(key);
			textFields.add(temp);
			this.doublePanel.add(new JLabel(key));
			this.doublePanel.add(temp);
		}
	}

	public void writeDoubleParameter() {
		for (JTextField textField : textFields) {
			GeneticAlgorithm.doubleParameter.put(textField.getName(), Double
					.parseDouble(textField.getText()));
		}
		GeneticAlgorithm.setDoubleParameter();
	}
}

@SuppressWarnings("serial")
class Display extends JPanel {
	public static final String GRAPH = "1", GRID = "2",
			SELECTION_PROBABILITY = "3", LOCAL_DIVERSITY = "4",
			TEST_SELECTIVE_PRESSURE = "5";

	public static int gridHeightOffset = 50;

	public ExcelHandler excelHandler;

	private Dimension gridDimension;

	private int graphWidth, graphHeight;

	private JFreeChart chart;

	private XYPlot plot;

	private XYDataset dataset1, dataset2;

	private XYLineAndShapeRenderer renderer;

	private NumberAxis domainAxis, rangeAxis;

	private ScatterXYItemLabelGenerator generator;

	private BufferedImage image;

	private JPanel graphPanel, graphContainer, gridContainer, statusPanel,
			selectionProbabitlityContainer, localDiversityContainer,
			testSelectivePressureContainer;

	private GridFitnessLandscapeDisplay gridFitnessLandscapePanel;

	private GridRatioDisplay selectionProbabilityPanel, localDiversityPanel,
			testSelectivePressurePanel;

	public boolean autoZoom = true, enableMouseWheelZoom = false,
			shrink = true, square = true;

	private JLabel lblChart, lblBestSoFar, lblEvaluationCount, lblGeneration;

	public boolean isDisplayDiversity = GeneticAlgorithm.SHOW_DIVERSITY,
			isDisplayMating = GeneticAlgorithm.SHOW_MATING_PROB
					|| GeneticAlgorithm.SHOW_MATING_COUNT,
			isDisplayOptimalParameter = GeneticAlgorithm.SHOW_OPTIMAL_PARAMETER;

	public Display(int graphWidth, int graphHeight, ExcelHandler excelHandler) {
		super();
		this.graphWidth = graphWidth;
		this.graphHeight = graphHeight;
		this.setPreferredSize(new Dimension(graphWidth, graphHeight));
		lblChart = new JLabel();
		lblBestSoFar = new JLabel();
		lblEvaluationCount = new JLabel();
		lblGeneration = new JLabel();
		lblChart.addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent arg0) {
				if (enableMouseWheelZoom) {
					double zoomCoEff = 1;
					if (arg0.getWheelRotation() == -1)
						zoomCoEff = 0.5;
					else if (arg0.getWheelRotation() == 1)
						zoomCoEff = 2;
					zoomGraph(zoomCoEff, arg0.getX(), arg0.getY());
				}
			}
		});
		lblChart.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				autoZoom = !autoZoom;
				if (autoZoom) {
					domainAxis.setAutoRange(true);
					rangeAxis.setAutoRange(true);
				} else {
					domainAxis.setAutoRange(false);
					rangeAxis.setAutoRange(false);
				}
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

			public void mousePressed(MouseEvent e) {
			}

			public void mouseReleased(MouseEvent e) {
			}
		});

		graphPanel = new JPanel();
		graphPanel.add(lblChart);
		graphContainer = new JPanel();
		graphContainer
				.setLayout(new BoxLayout(graphContainer, BoxLayout.Y_AXIS));
		statusPanel = new JPanel(new FlowLayout());
		statusPanel.add(lblBestSoFar);
		statusPanel.add(lblGeneration);
		statusPanel.add(lblEvaluationCount);
		graphContainer.add(graphPanel);
		graphContainer.add(statusPanel);
		gridContainer = new JPanel();
		gridContainer.setLayout(new BoxLayout(gridContainer, BoxLayout.Y_AXIS));
		gridFitnessLandscapePanel = new GridFitnessLandscapeDisplay();
		gridContainer.add(gridFitnessLandscapePanel);
		selectionProbabitlityContainer = new JPanel();
		selectionProbabitlityContainer.setLayout(new BoxLayout(
				selectionProbabitlityContainer, BoxLayout.Y_AXIS));
		selectionProbabilityPanel = new GridRatioDisplay();
		selectionProbabitlityContainer.add(selectionProbabilityPanel);
		localDiversityContainer = new JPanel();
		localDiversityContainer.setLayout(new BoxLayout(
				localDiversityContainer, BoxLayout.Y_AXIS));
		localDiversityPanel = new GridRatioDisplay();
		localDiversityContainer.add(localDiversityPanel);
		testSelectivePressureContainer = new JPanel();
		testSelectivePressureContainer.setLayout(new BoxLayout(
				testSelectivePressureContainer, BoxLayout.Y_AXIS));
		testSelectivePressurePanel = new GridRatioDisplay();
		testSelectivePressureContainer.add(testSelectivePressurePanel);
		this.setLayout(new CardLayout());
		this.add(graphContainer, GRAPH);
		this.add(gridContainer, GRID);
		this.add(selectionProbabitlityContainer, SELECTION_PROBABILITY);
		this.add(localDiversityContainer, LOCAL_DIVERSITY);
		this.add(testSelectivePressureContainer, TEST_SELECTIVE_PRESSURE);
		this.excelHandler = excelHandler;

		int min = (int) Math.min(graphWidth, graphHeight + gridHeightOffset);
		if (square)
			gridDimension = new Dimension(min, min);
		else
			gridDimension = new Dimension(graphWidth, graphHeight
					+ gridHeightOffset);
	}

	public void initXYGraph() {
		graphContainer.add(statusPanel);
		dataset1 = new XYSeriesCollection();
		((XYSeriesCollection) dataset1).addSeries(new XYSeries("Best"));
		((XYSeriesCollection) dataset1).addSeries(new XYSeries("Mean"));

		if (isDisplayDiversity) {
			dataset2 = new XYSeriesCollection();
			((XYSeriesCollection) dataset2).addSeries(new XYSeries(
					"Global Diversity"));
		} else if (isDisplayOptimalParameter) {
			dataset2 = new XYSeriesCollection();
			((XYSeriesCollection) dataset2).addSeries(new XYSeries(
					"Optimal Parameter"));
		} else if (isDisplayMating) {
			dataset2 = new XYSeriesCollection();
			((XYSeriesCollection) dataset2).addSeries(new XYSeries("P00"));
			((XYSeriesCollection) dataset2).addSeries(new XYSeries("P01"));
			((XYSeriesCollection) dataset2).addSeries(new XYSeries("P11"));
		}

		chart = ChartFactory.createXYLineChart("Single Objective", // Title
				"Generation", // x-axis Label
				"Best Fitness Value", // y-axis Label
				dataset1, // DataSet
				PlotOrientation.VERTICAL, // Plot Orientation
				true, // Show Legend
				true, // Use ToolTips
				false // Generate URLs
				);

		XYPlot plot = chart.getXYPlot();

		if (isDisplayMating) {
			ValueAxis[] axis = new ValueAxis[1];
			if (GeneticAlgorithm.SHOW_MATING_PROB)
				axis[0] = new LogarithmicAxis("Success Mating Probabitlity");
			else if (GeneticAlgorithm.SHOW_MATING_COUNT)
				axis[0] = new LogarithmicAxis("Expected Mating Count");
			plot.setRangeAxes(axis);
			plot.setDataset(0, dataset2);
		} else if (isDisplayDiversity || isDisplayOptimalParameter) {
			ValueAxis[] axis = new ValueAxis[2];
			axis[0] = new NumberAxis("Best Fitness Value");
			//axis[0].setRange(5500, 8500);
			if (isDisplayDiversity)
				axis[1] = new NumberAxis("Diversity");
			else if (isDisplayOptimalParameter)
				axis[1] = new NumberAxis("Optimal Parameter");
			plot.setRangeAxes(axis);
			plot.setDataset(1, dataset2);
			plot.mapDatasetToRangeAxis(1, 1);
			plot.setRenderer(1, new XYLineAndShapeRenderer());
			XYItemRenderer ren = plot.getRendererForDataset(dataset2);
			((XYLineAndShapeRenderer) ren).setSeriesShapesVisible(0, false);
			ren.setSeriesPaint(0, Color.GREEN);
		}
	}

	public void initScatterGraph(double[] lowerRange, double[] upperRange) {
		graphContainer.add(statusPanel);
		dataset1 = new XYSeriesCollection();
		((XYSeriesCollection) dataset1).addSeries(new XYSeries("Fitness Rank",
				false));

		chart = ChartFactory.createScatterPlot("Multiple Objective", "Fn1",
				"Fn2", dataset1, PlotOrientation.VERTICAL, true, true, false);
		plot = chart.getXYPlot();
		domainAxis = (NumberAxis) plot.getDomainAxis();
		rangeAxis = (NumberAxis) plot.getRangeAxis();
		if (!autoZoom) {
			domainAxis.setRange(lowerRange[0], upperRange[0]);
			rangeAxis.setRange(lowerRange[1], upperRange[1]);
		}
		renderer = (XYLineAndShapeRenderer) plot.getRenderer();
		generator = new ScatterXYItemLabelGenerator();
		renderer.setBasePositiveItemLabelPosition(new ItemLabelPosition(
				ItemLabelAnchor.OUTSIDE2, TextAnchor.BOTTOM_LEFT));
		renderer.setBaseItemLabelGenerator(generator);
		renderer.setBaseItemLabelsVisible(true);
	}

	public void zoomGraph(double zoomCoEff, int x, int y) {
		int xOffset = 40, yLowerOffset = 65, yUpperOffset = 27;
		if (x < xOffset && y < graphHeight - yLowerOffset) {
			double upperBound = rangeAxis.getRange().getUpperBound();
			double lowerBound = rangeAxis.getRange().getLowerBound();
			double yValue = (upperBound - lowerBound)
					/ (graphHeight - yLowerOffset - yUpperOffset)
					* (graphHeight - yLowerOffset - y) + lowerBound;
			setRangeAxis(zoomCoEff, yValue);
		}
		if (y > graphHeight - yLowerOffset && x > xOffset) {
			double upperBound = domainAxis.getRange().getUpperBound();
			double lowerBound = domainAxis.getRange().getLowerBound();
			double xValue = (upperBound - lowerBound) / (graphWidth - xOffset)
					* (x - xOffset) + lowerBound;
			setDomainAxis(zoomCoEff, xValue);
		}
		updateGraphImage();
	}

	public void setDomainAxis(double zoomCoEff, double xValue) {
		double upperBound = domainAxis.getRange().getUpperBound();
		double lowerBound = domainAxis.getRange().getLowerBound();
		domainAxis.setRange(xValue - ((xValue - lowerBound) * zoomCoEff),
				xValue + ((upperBound - xValue) * zoomCoEff));
	}

	public void setRangeAxis(double zoomCoEff, double yValue) {
		double upperBound = rangeAxis.getRange().getUpperBound();
		double lowerBound = rangeAxis.getRange().getLowerBound();
		rangeAxis.setRange(yValue - ((yValue - lowerBound) * zoomCoEff), yValue
				+ ((upperBound - yValue) * zoomCoEff));
	}

	@SuppressWarnings("unchecked")
	public void clearGraphValue(int serieIndex) {
		List series = ((XYSeriesCollection) dataset1).getSeries();
		((XYSeries) series.get(serieIndex)).clear();
	}

	@SuppressWarnings("unchecked")
	public void updateGraphValue(int dataset, int serieIndex, double x, double y) {
		List series = null;
		switch (dataset) {
		case 0:
			series = ((XYSeriesCollection) dataset1).getSeries();
			break;
		case 1:
			series = ((XYSeriesCollection) dataset2).getSeries();
			break;
		default:
			series = ((XYSeriesCollection) dataset1).getSeries();
		}
		((XYSeries) series.get(serieIndex)).add(x, y);
	}

	public void initialiseGridData(int[][] data, int range) {
		gridFitnessLandscapePanel.initialiseGridData(data, range);
		if (shrink) {
			gridFitnessLandscapePanel.setShrinkSize(gridDimension);
		} else {
			// to make scroll bar appear
			Dimension d = new Dimension(
					(int) (gridFitnessLandscapePanel.xSpace
							* gridFitnessLandscapePanel.xSize * (data.length + gridFitnessLandscapePanel.xOffset)),
					(int) (gridFitnessLandscapePanel.ySpace
							* gridFitnessLandscapePanel.ySize * (data[0].length + gridFitnessLandscapePanel.yOffset)));
			this.setPreferredSize(d);
			d = new Dimension(
					(int) ((data.length + gridFitnessLandscapePanel.xOffset)
							* gridFitnessLandscapePanel.xSpace * gridFitnessLandscapePanel.xSize),
					gridFitnessLandscapePanel.getMaximumSize().height);
			gridFitnessLandscapePanel.setMaximumSize(d);
		}
	}

	public void initialisePopulation(int popSize) {
		gridFitnessLandscapePanel.initialisePopulation(popSize);
	}

	public void clearGridValue() {
		gridFitnessLandscapePanel.resetGridData();
	}

	public void updateGridValue(int x, int y) {
		updateGridValue(x, y, GridFitnessLandscapeDisplay.POPULATION_TYPE);
	}

	public void updateGridValue(int x, int y, int type) {
		if (type == GridFitnessLandscapeDisplay.BLANK_TYPE)
			gridFitnessLandscapePanel.setBlank(x, y);
		else if (type == GridFitnessLandscapeDisplay.BEST_SO_FAR_TYPE)
			gridFitnessLandscapePanel.setBestSoFar(x, y);
		else if (type == GridFitnessLandscapeDisplay.ACTIVE_TYPE)
			gridFitnessLandscapePanel.setActive(x, y);
		else if (type == GridFitnessLandscapeDisplay.PRODUCT_TYPE)
			gridFitnessLandscapePanel.setProduct(x, y);
		else if (type == GridFitnessLandscapeDisplay.NEXT_GENERATION_TYPE)
			gridFitnessLandscapePanel.setNextGeneration(x, y);
		else
			gridFitnessLandscapePanel.setPopulation(x, y);
	}

	public void updateActiveIndex(int activeIndex) {
		gridFitnessLandscapePanel.activeIndex = activeIndex;
	}

	public void updatePopulation(ArrayList<Genome<int[]>> population) {
		for (int i = 0; i < population.size(); i++) {
			Genome<int[]> g = population.get(i);
			gridFitnessLandscapePanel.setPopulationData(i, g.chromosome[0],
					g.chromosome[1]);
		}
	}

	public void updateGraphImage() {
		try {
			image = chart.createBufferedImage(graphWidth, graphHeight);
			lblChart.setIcon(new ImageIcon(image));
		} catch (IllegalArgumentException e) {
		}
	}

	public void updateAlgorithmStatus(int generationNo) {
		updateAlgorithmStatus(null, generationNo, null);
	}

	public void updateAlgorithmStatus(int generationNo, int evaluationCount) {
		updateAlgorithmStatus(null, generationNo, evaluationCount);
	}

	public void updateAlgorithmStatus(double bestSoFar, int evaluationCount) {
		updateAlgorithmStatus(bestSoFar, null, evaluationCount);
	}

	public void updateAlgorithmStatus(Double bestSoFar, Integer generationNo,
			Integer evaluationCount) {
		if (bestSoFar != null)
			lblBestSoFar.setText(" Best So Far : " + bestSoFar);
		else
			lblBestSoFar.setText("");
		if (generationNo != null)
			lblGeneration.setText(" Generation : " + generationNo);
		else
			lblGeneration.setText("");
		if (evaluationCount != null)
			lblEvaluationCount
					.setText(" Evaluation Count : " + evaluationCount);
		else
			lblEvaluationCount.setText("");
	}

	public void setOptimizer(MultiObjectiveRealCodedGeneticAlgorithm morcga) {
		this.generator.setOptimizer(morcga);
	}

	public ImageIcon getGraphImage() {
		image = chart.createBufferedImage(graphWidth, graphHeight);
		return new ImageIcon(image);
	}

	public void initializeSelectionProbability(double[][] data, double range) {
		selectionProbabilityPanel.initialiseGridData(data, range);
		selectionProbabilityPanel.setShrinkSize(gridDimension);
	}

	public void setSelectionProbability(int x, int y, double value) {
		selectionProbabilityPanel.setValue(x, y, value);
	}

	public void initializeLocalDiversity(double[][] data, double range) {
		localDiversityPanel.reverse = true;
		localDiversityPanel.initialiseGridData(data, range);
		localDiversityPanel.setShrinkSize(gridDimension);
	}

	public void setLocalDiversity(int x, int y, double value) {
		localDiversityPanel.setValue(x, y, value);
	}

	public void setRange(double range) {
		localDiversityPanel.setRange(range);
	}

	public void initialTestSelectivePressure(double[][] data, double range) {
		testSelectivePressurePanel.reverse = true;
		testSelectivePressurePanel.initialiseGridData(data, range);
		testSelectivePressurePanel.setShrinkSize(gridDimension);
		testSelectivePressureContainer.add(statusPanel);
	}

	public void setBestIndividual(int x, int y, double value) {
		testSelectivePressurePanel.setValue(x, y, value);
	}
}

class ScatterXYItemLabelGenerator implements XYItemLabelGenerator {
	private MultiObjectiveRealCodedGeneticAlgorithm morcga;

	public String generateLabel(XYDataset dataset, int series, int item) {
		return "" + morcga.getRank(item);
	}

	public void setOptimizer(MultiObjectiveRealCodedGeneticAlgorithm morcga) {
		this.morcga = morcga;
	}
}

@SuppressWarnings("serial")
class GeneralGridDisplay extends JPanel {

	public static Color BLANK_COLOR = Color.WHITE;

	public int xSize = 20, ySize = 20, xOffset = 0, yOffset = 0;

	public double range, xSpace = 1.25, ySpace = 1.25;

	protected GridData[][] data;

	public void initialiseGridData(int[][] data, int range) {
		this.range = range;
		this.data = new GridData[data.length][data[0].length];
		for (int i = 0; i < data.length; i++)
			for (int j = 0; j < data[0].length; j++)
				this.data[i][j] = new GridData(data[i][j] + "", BLANK_COLOR);
	}

	public void setShrinkSize(Dimension d) {
		xSize = (int) (d.getWidth() / (data.length + xOffset) / xSpace);
		ySize = (int) (d.getHeight() / (data[0].length + yOffset) / ySpace);
		d = new Dimension((int) ((data.length + xOffset) * xSpace * xSize),
				(int) ((data[0].length + yOffset) * ySpace * ySize));
		this.setPreferredSize(d);
		d.setSize(d.getWidth(), this.getMaximumSize().getHeight());
		this.setMaximumSize(d);
	}

	public void resetGridData() {
		for (int i = 0; i < data.length; i++)
			for (int j = 0; j < data[0].length; j++)
				setBlank(i, j);
	}

	public void setBlank(int x, int y) {
		data[x][y].setMainColor(BLANK_COLOR);
		data[x][y].setSubColor(null);
	}

	public void paint(Graphics g) {
		super.paint(g);
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		for (int i = 0; i < data.length; i++)
			for (int j = 0; j < data[0].length; j++) {
				g2.setPaint(data[i][j].mainColor);
				g2.fill(new Rectangle2D.Double((i + xOffset) * xSpace * xSize,
						(j + yOffset) * ySpace * ySize, xSize, ySize));
				if (data[i][j].subColor != null) {
					g2.setPaint(data[i][j].subColor);
					g2.fill(new Rectangle2D.Double((i + xOffset) * xSpace
							* xSize, (j + yOffset) * ySpace * ySize, xSize / 2,
							ySize / 2));
				}
				g2.setPaint(Color.BLACK);
				g2.drawString(data[i][j].text,
						(int) ((i + xOffset) * xSpace * xSize),
						(int) ((j + yOffset) * ySpace * ySize + ySize));
			}
	}

	public static Color getGreyScaleColor(double value, double range,
			boolean reverse) {
		if (range < value)
			System.err.println("Incorrect grid display");
		if (reverse)
			value = range - value;
		return Color.getHSBColor(0, 0, (float) (value / range));
	}
}

@SuppressWarnings("serial")
class GridRatioDisplay extends GeneralGridDisplay {

	public boolean reverse = false;

	public void initialiseGridData(double[][] data, double range) {
		this.range = range;
		this.data = new GridData[data.length][data[0].length];
		for (int i = 0; i < data.length; i++)
			for (int j = 0; j < data[0].length; j++)
				this.data[i][j] = new GridData("", GeneralGridDisplay
						.getGreyScaleColor(data[i][j], range, reverse));
	}

	public void setValue(int x, int y, double value) {
		this.data[x][y].setMainColor(GeneralGridDisplay.getGreyScaleColor(
				value, range, reverse));
	}

	public void setRange(double range) {
		this.range = range;
	}
}

@SuppressWarnings("serial")
class GridFitnessLandscapeDisplay extends GeneralGridDisplay {
	public static int BLANK_TYPE = 0, POPULATION_TYPE = 1;

	public static int BEST_SO_FAR_TYPE = 2, ACTIVE_TYPE = 3, PRODUCT_TYPE = 4,
			NEXT_GENERATION_TYPE = 5;

	public static Color BEST_SO_FAR_COLOR = Color.GREEN,
			POPULATION_COLOR = Color.RED, ACTIVE_COLOR = Color.CYAN,
			PRODUCT_COLOR = Color.ORANGE, NEXT_GENERATION_COLOR = Color.ORANGE;

	public int activeIndex = 0;

	public int[][] population;

	public GridFitnessLandscapeDisplay() {
		super();
		this.yOffset = 3;
	}

	public void initialiseGridData(int[][] data, int range) {
		super.initialiseGridData(data, range);
		initialisePopulation(0);
	}

	public void initialisePopulation(int popSize) {
		this.population = new int[popSize][2];
	}

	public void setPopulationData(int index, int x, int y) {
		this.population[index][0] = x;
		this.population[index][1] = y;
	}

	public void setPopulation(int x, int y) {
		data[x][y].setMainColor(POPULATION_COLOR);
	}

	public void setBestSoFar(int x, int y) {
		data[x][y].setMainColor(BEST_SO_FAR_COLOR);
	}

	public void setActive(int x, int y) {
		data[x][y].setMainColor(ACTIVE_COLOR);
	}

	public void setProduct(int x, int y) {
		data[x][y].setMainColor(PRODUCT_COLOR);
	}

	public void setNextGeneration(int x, int y) {
		data[x][y].setSubColor(NEXT_GENERATION_COLOR);
	}

	public void paint(Graphics g) {
		super.paint(g);
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		if (GeneticAlgorithm.STEP_DETAIL) {
			int borderWidth = 2;
			for (int i = 0; i < population.length; i++) {
				double d = Double
						.parseDouble(data[population[i][0]][population[i][1]].text);
				if (i == activeIndex) {
					g2.setPaint(Color.GREEN);
					g2.fill(new Rectangle2D.Double((i + xOffset) * xSpace
							* xSize, ySpace * ySize, xSize, ySize));
					g2.setPaint(GeneralGridDisplay.getGreyScaleColor(d, range,
							false));
					g2.fill(new Rectangle2D.Double((i + xOffset) * xSpace
							* xSize + borderWidth,
							ySpace * ySize + borderWidth, xSize - 2
									* borderWidth, ySize - 2 * borderWidth));
				} else {
					g2.setPaint(GeneralGridDisplay.getGreyScaleColor(d, range,
							false));
					g2.fill(new Rectangle2D.Double((i + xOffset) * xSpace
							* xSize, ySpace * ySize, xSize, ySize));
				}
			}

			for (int i = 0; i < population.length; i++) {
				g2.setPaint(Color.BLACK);
				g2.drawLine((int) ((i + xOffset + 0.5) * xSpace * xSize),
						(int) (1.5 * ySpace * ySize), (int) ((population[i][0]
								+ xOffset + 0.5)
								* xSpace * xSize), (int) ((population[i][1]
								+ yOffset + 0.3)
								* ySpace * ySize));
			}
		}
	}
}

class GridData {
	public String text;
	public Color mainColor;
	public Color subColor;

	public GridData(String text, Color color) {
		this.text = text;
		this.mainColor = color;
	}

	public void setText(String text) {
		this.text = text;
	}

	public void setMainColor(Color color) {
		this.mainColor = color;
	}

	public void setSubColor(Color color) {
		this.subColor = color;
	}
}

class ExcelHandler {
	public static String EXCEL_OUTPUT_SUFFIX = "";

	private static String EXCEL_OUTPUT_PATH = "." + GUI.fs
			+ "Experimental Results" + EXCEL_OUTPUT_SUFFIX + ".xls";

	private static String GRAPH_OUTPUT_PATH = "." + GUI.fs
			+ "Experimental Graph" + GUI.fs;

	public static boolean WRITING = false;

	private SimpleDateFormat simpleDateFormat = new SimpleDateFormat(
			"yyyyMMdd_HHmmss");

	private DateFormat customDateFormat = new DateFormat("dd MMM yyyy hh:mm:ss");

	private Date now;

	public static int NORMAL = 0, GRID = 1, SELECTIVE_PRESSURE = 2,
			MATING_PROB = 3;

	public static void updatePath() {
		EXCEL_OUTPUT_PATH = "." + GUI.fs + "Experimental Results"
				+ EXCEL_OUTPUT_SUFFIX + ".xls";
	}

	public synchronized void printResults(GeneticAlgorithm<?> ga,
			int resultType, boolean printGraph) {
		Workbook workbook;
		WritableWorkbook copy;
		while (WRITING) {
			System.out.println("wait");
			try {
				Thread.sleep(10000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		try {
			WRITING = true;
			System.out.println("read");
			workbook = Workbook.getWorkbook(new File(EXCEL_OUTPUT_PATH));
			copy = Workbook.createWorkbook(new File(EXCEL_OUTPUT_PATH),
					workbook);
			WritableSheet sheet = copy.getSheet(0);
			int i = 1, j = 0;
			while (i < 1000000) {
				if (checkEmptyRow(sheet, i)) {
					int k = i / 2;
					while (i - k != 1) {
						if (checkEmptyRow(sheet, (k + i) / 2))
							i = (k + i) / 2;
						else
							k = (k + i) / 2;
					}
					break;
				}
				i *= 2;
			}
			now = Calendar.getInstance().getTime();
			WritableCellFormat dateFormat = new WritableCellFormat(
					customDateFormat);
			String fileName = GRAPH_OUTPUT_PATH + simpleDateFormat.format(now)
					+ ".jpg";
			DateTime dateCell = new DateTime(j++, i, now, dateFormat);
			sheet.addCell(dateCell);
			Label label = new Label(j++, i, ga.PROBLEM_NAME);
			sheet.addCell(label);
			Number number = new Number(j++, i, ga.SEED);
			sheet.addCell(number);
			// ***************************
			label = new Label(j++, i,
					GeneticAlgorithm.SELECTION_METHOD_NAME[ga.SELECTION_METHOD]);
			sheet.addCell(label);
			// ***************************
			switch (ga.SELECTION_METHOD) {
			case 0:
				j++;
				break;
			case 1:
				number = new Number(j++, i, ga.POOL_SIZE);
				sheet.addCell(number);
				break;
			case 2:
				number = new Number(j++, i, ga.SELECTIVE_PROBABILITY);
				sheet.addCell(number);
				break;
			default:
				j++;
				break;
			}
			// ***************************
			label = new Label(j++, i,
					GeneticAlgorithm.STRUCTURE_NAME[ga.STRUCTURE]);
			sheet.addCell(label);
			// ***************************
			if (ga.STRUCTURE == GeneticAlgorithm.CELLULAR2D) {
				label = new Label(j++, i, ga.populationSizeX + " x "
						+ ga.populationSizeY);
				sheet.addCell(label);
			} else {
				number = new Number(j++, i, ga.populationSize);
				sheet.addCell(number);
			}
			// ***************************
			if (ga.STRUCTURE == GeneticAlgorithm.CELLULAR1D) {
				label = new Label(j++, i, BitStringGenome
						.printArray(ga.selectionPattern));
				sheet.addCell(label);
			} else if (ga.STRUCTURE == GeneticAlgorithm.CELLULAR2D) {
				if (ga.NEIGHBORHOOD_SHAPE == GeneticAlgorithm.LINEAR_NEIGHBORHOOD) {
					label = new Label(j++, i, "L"
							+ (ga.neighborhoodSize * 4 + 1));
					sheet.addCell(label);
				} else if (ga.NEIGHBORHOOD_SHAPE == GeneticAlgorithm.RHOMBUS_NEIGHBORHOOD) {
					// label = new Label(j++, i, "R"
					// + (int) (1 + 4 * (ga.neighborhoodSize * 0.5)
					// * (ga.neighborhoodSize + 1)));
					label = new Label(j++, i, "R " + ga.neighborhoodSize);
					sheet.addCell(label);
				} else
					j++;
			} else {
				j++;
			}
			// ***************************
			label = new Label(j++, i,
					GeneticAlgorithm.SELECTION_SCHEME_NAME[ga.SELECTION_SCHEME]);
			sheet.addCell(label);
			if (ga.SELECTION_SCHEME == GeneticAlgorithm.POLYNOMIAL_SELECTION) {
				number = new Number(j++, i, ga.POLYNOMIAL_PARAMETER);
				sheet.addCell(number);
			} else if (ga.SELECTION_SCHEME == GeneticAlgorithm.CENTRIC_SELECTION) {
				number = new Number(j++, i, ga.CENTRIC_PARAMETER);
				sheet.addCell(number);
			}
			String s = "one";
			if (ga.REPLACE_IF_BETTER) {
				s += " if better";
			} else {
				s = "always " + s;
			}
			label = new Label(j++, i, s);
			sheet.addCell(label);
			number = new Number(j++, i, ga.MUTATION_RATE);
			sheet.addCell(number);
			number = new Number(j++, i, ga.CROSSOVER_RATE);
			sheet.addCell(number);
			number = new Number(j++, i, ga.ELITE);
			sheet.addCell(number);
			number = new Number(j++, i, ga.chromosomeSize);
			sheet.addCell(number);
			number = new Number(j++, i, ga.generationSize);
			sheet.addCell(number);
			if (resultType == MATING_PROB) {
				int temp = j;
				label = new Label(j++, i, "00");
				sheet.addCell(label);
				for (int k = 0; k < ga.generationSize; k++) {
					number = new Number(j++, i, ga.arraySuccessMatingCount00[k]);
					sheet.addCell(number);
				}
				j = temp;
				label = new Label(j++, ++i, "01");
				sheet.addCell(label);
				for (int k = 0; k < ga.generationSize; k++) {
					number = new Number(j++, i, ga.arraySuccessMatingCount01[k]);
					sheet.addCell(number);
				}
				j = temp;
				label = new Label(j++, ++i, "11");
				sheet.addCell(label);
				for (int k = 0; k < ga.generationSize; k++) {
					number = new Number(j++, i, ga.arraySuccessMatingCount11[k]);
					sheet.addCell(number);
				}
			} else if (resultType == SELECTIVE_PRESSURE) {
				number = new Number(j++, i,
						((TestSelectivePressure) ga).takeOverTime);
				sheet.addCell(number);
				if (GeneticAlgorithm.PRINT_POPGROWTH)
					for (int k = 0; k < ga.generationSize; k++) {
						number = new Number(j++, i,
								((TestSelectivePressure) ga).arrayBestCount[k]);
						sheet.addCell(number);
					}
			} else {
				number = new Number(j++, i, ga.BEST_SO_FAR);
				sheet.addCell(number);
				number = new Number(j++, i, ga.generationUsed);
				sheet.addCell(number);
				number = new Number(j++, i, ga.evaluationCount);
				sheet.addCell(number);
			}
			// ***************************
			if (printGraph) {
				File graphFile = printGraphImage(ga.output, fileName,
						resultType);
				WritableHyperlink hyperlink = new WritableHyperlink(j++, i,
						graphFile);
				sheet.addHyperlink(hyperlink);
			}
			// ***************************
			copy.write();
			copy.close();
			workbook.close();
			WRITING = false;
			System.out.println("write");
		} catch (IOException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (RowsExceededException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (WriteException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (BiffException e) {
			e.printStackTrace();
			WRITING = false;
		}
	}

	public synchronized void printVerticleResults(GeneticAlgorithm<?> ga,
			int resultType) {
		int colNum = 4, headerRowNum = 4;
		while (WRITING) {
			System.out.println("wait");
			try {
				Thread.sleep(10000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		Workbook workbook;
		WritableWorkbook copy;
		try {
			System.out.println("read");
			WRITING = true;
			workbook = Workbook.getWorkbook(new File(EXCEL_OUTPUT_PATH));
			copy = Workbook.createWorkbook(new File(EXCEL_OUTPUT_PATH),
					workbook);
			WritableSheet sheet = copy.getSheet(0);
			int i = 0, j = 1;
			while (true && i < 256) {
				Cell cell = sheet.getCell(i, j);
				String content = cell.getContents();
				if (content == "")
					break;
				i++;
			}
			Label label;
			Number number;
			if (resultType == SELECTIVE_PRESSURE) {
				label = new Label(i, j++, ga.PROBLEM_NAME);
				sheet.addCell(label);
				number = new Number(i, j++, ga.SEED);
				sheet.addCell(number);
				number = new Number(i, j++, ga.POLYNOMIAL_PARAMETER);
				sheet.addCell(number);
				label = new Label(i, j++, "R " + ga.neighborhoodSize);
				sheet.addCell(label);
			} else if (resultType == MATING_PROB) {
				for (int k = 0; k < colNum; k++, j -= headerRowNum) {
					label = new Label(i, j++, ga.PROBLEM_NAME);
					sheet.addCell(label);
					number = new Number(i, j++, ga.SEED);
					sheet.addCell(number);
					number = new Number(i, j++, ga.POLYNOMIAL_PARAMETER);
					sheet.addCell(number);
					label = new Label(i++, j++, "R " + ga.neighborhoodSize);
					sheet.addCell(label);
				}
				i -= colNum;
				j += headerRowNum;
				label = new Label(i++, j, "Best Count");
				sheet.addCell(label);
				// label = new Label(i++, j, "Global Diversity");
				// sheet.addCell(label);
				label = new Label(i++, j, "00");
				sheet.addCell(label);
				label = new Label(i++, j, "01");
				sheet.addCell(label);
				label = new Label(i++, j++, "11");
				sheet.addCell(label);
				i -= colNum;
			}
			for (int k = 0; k < ga.generationSize; k++) {
				if (resultType == SELECTIVE_PRESSURE) {
					number = new Number(i, j++,
							((TestSelectivePressure) ga).arrayBestCount[k]);
					sheet.addCell(number);
				} else if (resultType == MATING_PROB) {
					number = new Number(i++, j, ga.arrayBestCount[k]);
					sheet.addCell(number);
					// number = new Number(i++, j, ga.arrayGlobalDiversity[k]);
					// sheet.addCell(number);
					number = new Number(i++, j, ga.arraySuccessMatingCount00[k]);
					sheet.addCell(number);
					number = new Number(i++, j, ga.arraySuccessMatingCount01[k]);
					sheet.addCell(number);
					number = new Number(i++, j++,
							ga.arraySuccessMatingCount11[k]);
					sheet.addCell(number);
					i -= colNum;
				}
			}
			copy.write();
			copy.close();
			workbook.close();
			WRITING = false;
			System.out.println("write");
		} catch (IOException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (RowsExceededException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (WriteException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (BiffException e) {
			e.printStackTrace();
			WRITING = false;
		}
	}

	public synchronized void printOnlyResults(GeneticAlgorithm<?> ga) {
		Workbook workbook;
		WritableWorkbook copy;
		while (WRITING) {
			System.out.println("wait");
			try {
				Thread.sleep(10000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		try {
			WRITING = true;
			System.out.println("read");
			workbook = Workbook.getWorkbook(new File(EXCEL_OUTPUT_PATH));
			copy = Workbook.createWorkbook(new File(EXCEL_OUTPUT_PATH),
					workbook);
			WritableSheet sheet = copy.getSheet(0), parameterSheet = copy
					.getSheet(2);
			int i = 1, j = 0;
			while (i < 1000000) {
				if (checkEmptyRow(sheet, i)) {
					int k = i / 2;
					while (i - k != 1) {
						if (checkEmptyRow(sheet, (k + i) / 2))
							i = (k + i) / 2;
						else
							k = (k + i) / 2;
					}
					break;
				}
				i *= 2;
			}

			Label label = new Label(0, 2, "Neighborhood Size");
			parameterSheet.addCell(label);
			label = new Label(1, 2, "R " + ga.neighborhoodSize);
			parameterSheet.addCell(label);
			label = new Label(0, 3, "Probability Parameter");
			parameterSheet.addCell(label);
			Number number = new Number(1, 3, ga.POLYNOMIAL_PARAMETER);
			parameterSheet.addCell(number);

			for (int k = 0; k < ga.generationSize; k++) {
				number = new Number(j++, i, ga.arrayBestCount[k]);
				sheet.addCell(number);
				number = new Number(j++, i, ga.arraySuccessMatingCount00[k]);
				sheet.addCell(number);
				number = new Number(j++, i, ga.arraySuccessMatingCount01[k]);
				sheet.addCell(number);
				number = new Number(j++, i++, ga.arraySuccessMatingCount11[k]);
				sheet.addCell(number);
				j = 0;
			}
			copy.write();
			copy.close();
			workbook.close();
			WRITING = false;
			System.out.println("write");
		} catch (IOException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (RowsExceededException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (WriteException e) {
			e.printStackTrace();
			WRITING = false;
		} catch (BiffException e) {
			e.printStackTrace();
			WRITING = false;
		}
	}

	public File printGraphImage(Display output, String fileName, int graphType) {
		BufferedImage buffered = null;
		Graphics2D g = null;
		if (graphType == NORMAL) {
			ImageIcon imageIcon = output.getGraphImage();
			Image image = imageIcon.getImage();
			buffered = new BufferedImage(imageIcon.getIconWidth(), imageIcon
					.getIconHeight(), BufferedImage.TYPE_INT_RGB);
			g = buffered.createGraphics();
			g.drawImage(image, 0, 0, null);
		} else if (graphType == GRID) {
			buffered = new BufferedImage(output.getWidth(), output.getHeight(),
					BufferedImage.TYPE_INT_RGB);
			g = buffered.createGraphics();
			output.paint(g);
		}
		g.dispose();
		RenderedImage rendered = buffered;
		File graphFile = new File(fileName);
		try {
			ImageIO.write(rendered, "JPEG", graphFile);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return graphFile;
	}

	public void printGraphImage(Display output, int graphType) {
		now = Calendar.getInstance().getTime();
		String fileName = GRAPH_OUTPUT_PATH + simpleDateFormat.format(now)
				+ ".jpg";
		printGraphImage(output, fileName, graphType);
	}

	public boolean checkEmptyRow(WritableSheet sheet, int rowIndex) {
		int checkColumnNum = 20;
		for (int i = 0; i < checkColumnNum; i++) {
			Cell cell = sheet.getCell(i, rowIndex);
			String content = cell.getContents();
			if (content != "")
				return false;
		}
		return true;
	}
}

class MatingProbability {
	private HashMap<Integer, double[]> matingProb = new HashMap<Integer, double[]>();

	public MatingProbability(String problemName) {
		String path = "." + GUI.fs + "QAP" + GUI.fs + problemName + "prob.txt";
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(path));
			String line;
			while ((line = reader.readLine()) != null) {
				String[] result = line.split("\\s");
				double[] prob = new double[3];
				for (int i = 1; i < 4; i++)
					prob[i - 1] = Double.parseDouble(result[i]);
				matingProb.put(Integer.parseInt(result[0]), prob);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public double getMatingProb(int bestIndividualCount, int type) {
		return matingProb.get(bestIndividualCount)[type];
	}
}
