import java.io.*;
import java.util.*;

public abstract class GeneticAlgorithm<T> implements Runnable {

	public final static String[] SELECTION_METHOD_NAME = { "Roullette Wheel",
			"Tournament", "Combined Rank" };

	public final static String[] SELECTION_SCHEME_NAME = {
			"Polynomial Selection", "Centric Selection" };

	public final static String[] STRUCTURE_NAME = { "No Structure",
			"Cellular 1D", "Cellular 2D" };

	public final static int NO_STRUCTURE = 0, CELLULAR1D = 1, CELLULAR2D = 2,
			UNIFORM_SELECTION = 2, CENTRIC_SELECTION = 1,
			POLYNOMIAL_SELECTION = 0, LINEAR_NEIGHBORHOOD = 0,
			RHOMBUS_NEIGHBORHOOD = 1, SQUARE_NEIGHBORHOOD = 2, type00 = 0,
			type01 = 1, type11 = 2, matingSampleSize = 10000,
			bestIndividualInterval = 25;

	public final static int STEP_SELECTION = 1, STEP_CROSSOVER = 2,
			STEP_MUTATION = 3;

	public Random random = new Random();

	public MatingProbability matingProbability;

	public long DEFAULT_SEED = (long) (10000.0 * 1);

	public static HashMap<String, Boolean> booleanParameter;

	public static HashMap<String, Double> doubleParameter;

	public static boolean LOG_RESULT = true, PRINT_GRAPH = false,
			STEP_DETAIL = false, USE_DEFAULT_SEED = false,
			USE_PREDEFINED_FUNCTION = false, SHOW_DIVERSITY = false,
			SHOW_MATING_PROB = false, SHOW_MATING_COUNT = false,
			PRINT_MATING_PROB = false, PRINT_MATING_COUNT = false,
			STEP_ALGORITHM = true, PRINT_POPGROWTH = false,
			PRINT_OPTIMAL_PARAMETER = false, SHOW_OPTIMAL_PARAMETER = false;

	public boolean ADAPTIVE_MUTATION = false, ADAPTIVE_PARAMETER = false,
			REPLACE_IF_BETTER = true, COUNTER_LOCK = false, STEP = false,
			SQUARE_POPULATION_GRID = true;

	public long SEED = DEFAULT_SEED;

	public int STRUCTURE = CELLULAR2D, SELECTION_SCHEME = POLYNOMIAL_SELECTION,
			NEIGHBORHOOD_SHAPE = RHOMBUS_NEIGHBORHOOD;

	public long stepNum = 0, nextStepNum = 1, optimizingTime = 0,
			selectParentTime = 0, selectParentCount = 0,
			createNeighborhoodTime = 0, createNeighborhoodCount = 0;

	public int SELECTION_METHOD = 1, CROSSOVER_METHOD = 0, MUTATION_METHOD = 0,
			POOL_SIZE = 2, RADIUS = 2, radius = RADIUS, GENERATION_WINDOW = 10,
			populationSizeX, populationSizeY, neighborhoodSize = 1;

	public static double INITIAL_CROSSOVER_RATE = 0.8,
			INITIAL_MUTAION_RATE = 0.2;

	public double CROSSOVER_RATE = INITIAL_CROSSOVER_RATE,
			MUTATION_RATE = INITIAL_MUTAION_RATE, ELITE = 0.0,
			SELECTIVE_PROBABILITY = 0.8, ADAPTIVE_MUTATION_RATE = 0.1,
			ADAPTIVE_FACTOR = 0.05, CENTRIC_PARAMETER = 0.2,
			POLYNOMIAL_PARAMETER = BitStringTestFunction.convertParamC2P(0.2);

	public int CHROMOSOME_SIZE, POPULATION_SIZE, GENERATION_SIZE;

	public int[] selectionPattern = { -2, -1, 0, 1, 2 }, arrayMatingCount00,
			arrayMatingCount01, arrayMatingCount11, arraySuccessMatingCount00,
			arraySuccessMatingCount01, arraySuccessMatingCount11,
			arrayBestCount;

	public double[] selectionProbability1D, arrayGlobalDiversity;

	public double[][] selectionProbability2D, localDiversity,
			selectionProbabilityMapOnPopulation2D;

	public String PROBLEM_NAME;

	protected Genome<T> bestGenome;

	protected double BEST_SO_FAR, MEAN_FITNESS, DIFF_MEAN_FITNESS = -1,
			GLOBAL_DIVERSITY = 0;

	protected int activeIndex = 0;

	protected volatile Thread t;

	protected int chromosomeSize, populationSize, generationSize,
			generationUsed, evaluationCount, matingCount00, matingCount01,
			matingCount11, successMatingCount00, successMatingCount01,
			successMatingCount11;

	protected double maxProbability = 0;

	protected boolean maximize;

	protected ArrayList<Genome<T>> population, tempPopulation, nextGeneration,
			tournament, nonBestList, bestList;

	protected ArrayList<ArrayList<Genome<T>>> population2D;

	protected Display output;

	protected int generationCount;

	public abstract void initPopulation();

	public abstract void initDefaultParameter();

	public abstract double getDiversity(ArrayList<Genome<T>> a,
			ArrayList<Genome<T>> b);

	public GeneticAlgorithm(Display output) {
		this.population = new ArrayList<Genome<T>>();
		this.nextGeneration = new ArrayList<Genome<T>>();
		this.output = output;
		SEED = System.currentTimeMillis();
		random = new Random(SEED);
		initDefaultParameter();
		setParameter(CHROMOSOME_SIZE, GENERATION_SIZE, POPULATION_SIZE);
	}

	// under implementation
	public static void initBooleanParameter() {
		booleanParameter = new HashMap<String, Boolean>();
		booleanParameter.put("Log result", LOG_RESULT);
		booleanParameter.put("Print graph", PRINT_GRAPH);
		booleanParameter.put("Use default seed", USE_DEFAULT_SEED);
		booleanParameter.put("Step algorithm", STEP_ALGORITHM);
		booleanParameter.put("Show diversity", SHOW_DIVERSITY);
	}

	// under implementation
	public static void setBooleanParameter() {
		LOG_RESULT = booleanParameter.get("Log result");
		PRINT_GRAPH = booleanParameter.get("Print graph");
		USE_DEFAULT_SEED = booleanParameter.get("Use default seed");
		STEP_ALGORITHM = booleanParameter.get("Step algorithm");
		SHOW_DIVERSITY = booleanParameter.get("Show diversity");
	}

	// under implementation
	public static void initDoubleParameter() {
		doubleParameter = new HashMap<String, Double>();
		doubleParameter.put("Mutation Rate", INITIAL_MUTAION_RATE);
		doubleParameter.put("Crossover Rate", INITIAL_CROSSOVER_RATE);
	}

	// under implementation
	public static void setDoubleParameter() {
		INITIAL_MUTAION_RATE = doubleParameter.get("Mutation Rate");
		INITIAL_CROSSOVER_RATE = doubleParameter.get("Crossover Rate");
	}

	public void setParameter(int chromosome, int generation, int population) {
		this.chromosomeSize = chromosome;
		this.generationSize = generation;
		this.populationSize = population;
		if (SQUARE_POPULATION_GRID)
			this.populationSizeX = (int) Math.sqrt(population);
		this.evaluationCount = 0;
		if (USE_DEFAULT_SEED) {
			SEED = DEFAULT_SEED;
			random.setSeed(SEED);
		}
		populationSizeY = populationSize / populationSizeX;
		if (STRUCTURE == CELLULAR2D)
			setSelectionProbability2D();
		else
			setSelectionProbability1D();

		arrayMatingCount00 = new int[generation];
		arrayMatingCount01 = new int[generation];
		arrayMatingCount11 = new int[generation];
		arraySuccessMatingCount00 = new int[generation];
		arraySuccessMatingCount01 = new int[generation];
		arraySuccessMatingCount11 = new int[generation];
		arrayBestCount = new int[generation];
		arrayGlobalDiversity = new double[generation];
	}

	public void setSelectionProbability1D() {
		selectionProbability1D = new double[neighborhoodSize * 4 + 1];
		if (SELECTION_SCHEME == POLYNOMIAL_SELECTION) {
			double sum = 0;
			for (int i = 0; i < selectionProbability1D.length; i++) {
				if (i == selectionProbability1D.length / 2) {
					selectionProbability1D[i] = Math.pow(neighborhoodSize + 1,
							POLYNOMIAL_PARAMETER);
				} else if (i < selectionProbability1D.length / 2) {
					selectionProbability1D[i] = Math.pow(
							(i % neighborhoodSize) + 1, POLYNOMIAL_PARAMETER);
				} else if (i > selectionProbability1D.length / 2) {
					selectionProbability1D[i] = Math.pow(neighborhoodSize
							- ((i - 1) % neighborhoodSize),
							POLYNOMIAL_PARAMETER);
				}
				sum += selectionProbability1D[i];
			}
			for (int i = 0; i < selectionProbability1D.length; i++) {
				selectionProbability1D[i] /= sum;
			}
		} else if (SELECTION_SCHEME == CENTRIC_SELECTION) {
			for (int i = 0; i < selectionProbability1D.length; i++) {
				if (i == selectionProbability1D.length / 2)
					selectionProbability1D[i] = CENTRIC_PARAMETER;
				else
					selectionProbability1D[i] = (1 - CENTRIC_PARAMETER)
							/ (selectionProbability1D.length - 1);
			}
		} else if (SELECTION_SCHEME == UNIFORM_SELECTION)
			for (int i = 0; i < selectionProbability1D.length; i++)
				selectionProbability1D[i] = 1.0 / selectionProbability1D.length;
	}

	public double[][] initializeSelectionProbability2D() {
		int neighborhoodSizeX = neighborhoodSize, neighborhoodSizeY = neighborhoodSize;
		if (neighborhoodSize > (populationSizeX - 1) / 2)
			neighborhoodSizeX = (populationSizeX - 1) / 2;
		if (neighborhoodSize > (populationSizeY - 1) / 2)
			neighborhoodSizeY = (populationSizeY - 1) / 2;
		return new double[2 * neighborhoodSizeX + 1][2 * neighborhoodSizeY + 1];
	}

	public void setSelectionProbability2D() {
		selectionProbability2D = initializeSelectionProbability2D();
		maxProbability = setSelectionProbability2D(selectionProbability2D);
		this.output.initializeSelectionProbability(selectionProbability2D,
				maxProbability);
	}

	public double setSelectionProbability2D(double[][] selectionProbability2D) {
		if (SELECTION_SCHEME == POLYNOMIAL_SELECTION)
			return setSelectionProbability2D(selectionProbability2D,
					POLYNOMIAL_PARAMETER);
		else if (SELECTION_SCHEME == CENTRIC_SELECTION)
			return setSelectionProbability2D(selectionProbability2D,
					CENTRIC_PARAMETER);
		else
			return setSelectionProbability2D(selectionProbability2D, 0);
	}

	public double setSelectionProbability2D(double[][] selectionProbability2D,
			double parameter) {
		for (int i = 0; i < selectionProbability2D.length; i++)
			for (int j = 0; j < selectionProbability2D[0].length; j++) {
				int x = Math.abs(i - selectionProbability2D.length / 2);
				int y = Math.abs(j - selectionProbability2D[0].length / 2);

				if (SELECTION_SCHEME == POLYNOMIAL_SELECTION) {
					if (NEIGHBORHOOD_SHAPE == SQUARE_NEIGHBORHOOD) {
						selectionProbability2D[i][j] = Math.pow(
								neighborhoodSize + 1 - (x + y), parameter);
					} else if (NEIGHBORHOOD_SHAPE == LINEAR_NEIGHBORHOOD) {
						if (i == selectionProbability2D.length / 2
								|| j == selectionProbability2D[0].length / 2)
							selectionProbability2D[i][j] = Math.pow(
									neighborhoodSize + 1 - (x + y), parameter);
						else
							selectionProbability2D[i][j] = 0.0;
					} else if (NEIGHBORHOOD_SHAPE == RHOMBUS_NEIGHBORHOOD) {
						if (x + y <= neighborhoodSize)
							selectionProbability2D[i][j] = Math.pow(
									neighborhoodSize + 1 - (x + y), parameter);
						else
							selectionProbability2D[i][j] = 0.0;
					}

				} else if (SELECTION_SCHEME == CENTRIC_SELECTION) {
					// Linear Neighborhood size 1 ONLY
					if (i == selectionProbability2D.length / 2
							&& j == selectionProbability2D[0].length / 2)
						selectionProbability2D[i][j] = parameter;
					else if (i == selectionProbability2D.length / 2
							|| j == selectionProbability2D[0].length / 2)
						selectionProbability2D[i][j] = (1 - parameter)
								/ (selectionProbability2D.length
										+ selectionProbability2D[0].length - 2);
					else
						selectionProbability2D[i][j] = 0.0;

				} else if (SELECTION_SCHEME == UNIFORM_SELECTION) {
					if (NEIGHBORHOOD_SHAPE == SQUARE_NEIGHBORHOOD)
						selectionProbability2D[i][j] = 1.0 / (selectionProbability2D.length * selectionProbability2D[0].length);
					else if (NEIGHBORHOOD_SHAPE == LINEAR_NEIGHBORHOOD) {
						if (i == selectionProbability2D.length / 2
								|| j == selectionProbability2D[0].length / 2)
							selectionProbability2D[i][j] = 1.0 / (selectionProbability2D.length
									+ selectionProbability2D[0].length - 1);
						else
							selectionProbability2D[i][j] = 0.0;
					} else if (NEIGHBORHOOD_SHAPE == RHOMBUS_NEIGHBORHOOD) {
						if (x + y <= neighborhoodSize)
							selectionProbability2D[i][j] = 1.0 / (1 + 4.0
									* (neighborhoodSize * 0.5)
									* (neighborhoodSize + 1));
						else
							selectionProbability2D[i][j] = 0.0;
					}
				}
			}
		return normalizeSelectionProbability(selectionProbability2D);
	}

	public double normalizeSelectionProbability(
			double[][] selectionProbability2D) {
		double sum = 0, max = 0;
		for (int i = 0; i < selectionProbability2D.length; i++)
			for (int j = 0; j < selectionProbability2D[0].length; j++)
				sum += selectionProbability2D[i][j];

		for (int i = 0; i < selectionProbability2D.length; i++)
			for (int j = 0; j < selectionProbability2D[0].length; j++) {
				selectionProbability2D[i][j] /= sum;
				if (selectionProbability2D[i][j] > max)
					max = selectionProbability2D[i][j];
			}
		return max;
	}

	public void setSelectionProbabilityMapOnPopulation2D() {
		int neighborhoodSizeX = neighborhoodSize, neighborhoodSizeY = neighborhoodSize;
		if (neighborhoodSize > (populationSizeX - 1) / 2)
			neighborhoodSizeX = (populationSizeX - 1) / 2;
		if (neighborhoodSize > (populationSizeY - 1) / 2)
			neighborhoodSizeY = (populationSizeY - 1) / 2;
		selectionProbabilityMapOnPopulation2D = new double[populationSizeX][populationSizeY];
		for (int i = -neighborhoodSizeX; i <= neighborhoodSizeX; i++) {
			for (int j = -neighborhoodSizeY; j <= neighborhoodSizeY; j++) {
				int x = (activeIndex / populationSizeY + i + populationSizeX)
						% populationSizeX;
				int y = (activeIndex % populationSizeY + j + populationSizeY)
						% populationSizeY;
				selectionProbabilityMapOnPopulation2D[x][y] = selectionProbability2D[neighborhoodSizeX
						+ i][neighborhoodSizeY + j];
			}
		}
	}

	public void select() {
		switch (SELECTION_METHOD) {
		case 0:
			roulletteWheelSelect();
			break;
		case 1:
			tournamentSelect();
			break;
		case 2:
			combinedRankSelect();
			break;
		default:
			tournamentSelect();
		}
	}

	public void reproduce(Genome<T> g1, Genome<T> g2) {
		nextGeneration.add(mate(g1, g2));
		activeIndex = getAdjustedActiveIndex(1);

		if (STEP_DETAIL)
			updateActiveIndex(activeIndex);
	}

	public Genome<T> mate(Genome<T> g1, Genome<T> g2) {
		Genome<T> f = population.get(activeIndex), betterChild;
		int type;

		if (isBestSoFar(g1) && isBestSoFar(g2)) {
			matingCount11++;
			type = type11;
		} else if (isBestSoFar(g1) || isBestSoFar(g2)) {
			matingCount01++;
			type = type01;
		} else {
			matingCount00++;
			type = type00;
		}
		g1 = g1.clone();
		g2 = g2.clone();
		betterChild = g1;

		g1.crossOver(g2, this.CROSSOVER_RATE);
		if (STEP_DETAIL)
			updateIndividualsOnDisplay(g1, g2, STEP_CROSSOVER);
		g1.mutate(this.MUTATION_RATE);
		g2.mutate(this.MUTATION_RATE);
		if (STEP_DETAIL)
			updateIndividualsOnDisplay(g1, g2, STEP_MUTATION);

		if (isBetter(g2, g1))
			betterChild = g2;

		if (REPLACE_IF_BETTER && isBetter(f, betterChild))
			betterChild = f;

		if (isBetter(g1, bestGenome) || isBetter(g2, bestGenome)) {
			if (type == type11)
				successMatingCount11++;
			else if (type == type01)
				successMatingCount01++;
			else if (type == type00)
				successMatingCount00++;
		}

		return betterChild;
	}

	public boolean isBetter(Genome<T> g1, Genome<T> g2) {
		return (maximize && g1.getFitnessValue() > g2.getFitnessValue())
				|| (!maximize && g1.getFitnessValue() < g2.getFitnessValue());
	}

	public boolean isBestSoFar(Genome<T> g) {
		return g.getFitnessValue() == BEST_SO_FAR;
	}

	public int isBestSoFarFunction(Genome<T> g) {
		if (isBestSoFar(g))
			return 1;
		else
			return 0;
	}

	public void roulletteWheelSelect() {
		try {
			Genome<T> g1 = getRoulletteWheelWinner();
			Genome<T> g2 = getRoulletteWheelWinner();
			if (STEP_DETAIL) {
				updateIndividualsOnDisplay(g1, g2, STEP_SELECTION);
			}
			reproduce(g1, g2);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Genome<T> getRoulletteWheelWinner() throws Exception {
		double fitnessSum = 0, fitnessRatioSum = 0;
		double rand = random.nextDouble();
		Genome<T> g = null;
		for (Genome<T> gt : population) {
			fitnessSum += gt.getFitnessValue();
		}
		if (fitnessSum == 0) {
			throw new Exception();
		}
		for (int i = 0; i < population.size(); i++) {
			Genome<T> gt = population.get(i);
			fitnessRatioSum += gt.getFitnessValue() / fitnessSum;
			if (fitnessRatioSum > rand) {
				g = gt.clone();
				break;
			}
		}
		return g;
	}

	public void tournamentSelect() {
		Genome<T> g1 = this.getTournamentWinner();
		Genome<T> g2 = this.getTournamentWinner();
		if (STEP_DETAIL) {
			updateIndividualsOnDisplay(g1, g2, STEP_SELECTION);
		}
		reproduce(g1, g2);
	}

	public Genome<T> getTournamentWinner() {
		ArrayList<Genome<T>> neighborhood1D;
		tournament = new ArrayList<Genome<T>>();
		createNeighborhoodCount++;
		long start = System.currentTimeMillis();
		if (STRUCTURE == CELLULAR2D) {
			setSelectionProbabilityMapOnPopulation2D();
			for (int i = 0; i < POOL_SIZE; i++)
				tournament.add(getIndividualFromNeighborhood2D());

		} else if (STRUCTURE == CELLULAR1D) {
			neighborhood1D = PopulationFactory.get1DNeighborhoodPopulation(
					population, activeIndex, selectionPattern);
			for (int i = 0; i < POOL_SIZE; i++)
				tournament.add(neighborhood1D.get(getRoulletteWheelIndex()));
		} else {
			neighborhood1D = population;
			for (int i = 0; i < POOL_SIZE; i++) {
				int index = (int) (random.nextDouble() * neighborhood1D.size());
				tournament.add(neighborhood1D.get(index));
			}
		}
		createNeighborhoodTime += System.currentTimeMillis() - start;
		return Collections.min(tournament, new GenomeComparator(maximize));
	}

	public int getRoulletteWheelIndex() {
		double rand = random.nextDouble(), sum = 0;
		int j = 0;
		while (j < selectionProbability1D.length) {
			sum += selectionProbability1D[j];
			if (sum > rand) {
				break;
			}
			j++;
		}
		return j;
	}

	public Genome<T> getIndividualFromNeighborhood2D() {
		selectParentCount++;
		long start = System.currentTimeMillis();
		double rand = random.nextDouble(), sum = 0;
		for (int i = 0; i < selectionProbabilityMapOnPopulation2D.length; i++) {
			for (int j = 0; j < selectionProbabilityMapOnPopulation2D[0].length; j++) {
				sum += selectionProbabilityMapOnPopulation2D[i][j];
				if (sum > rand) {
					selectParentTime += System.currentTimeMillis() - start;
					return population2D.get(i).get(j);
				}
			}
		}
		return null;
	}

	public void combinedRankSelect() {
		Genome<T> g1 = this.getCombinedRankWinner();
		Genome<T> g2 = this.getCombinedRankWinner();
		if (STEP_DETAIL) {
			updateIndividualsOnDisplay(g1, g2, STEP_SELECTION);
		}
		reproduce(g1, g2);
	}

	public Genome<T> getCombinedRankWinner() {
		sortByCombineRank();
		Genome<T> g = population.get(0).clone();
		for (int i = 0; i < population.size(); i++) {
			if (random.nextDouble() < SELECTIVE_PROBABILITY) {
				g = population.get(i).clone();
				break;
			}
		}
		return g;
	}

	public int getAdjustedActiveIndex(int offset) {
		return getAdjustedIndex(activeIndex, offset, populationSize);
	}

	public static int getAdjustedIndex(int index, int offset, int arraySize) {
		int result = index + offset;
		result += arraySize;
		result %= arraySize;
		return result;
	}

	public void assignFitnessRank() {
		Collections.sort(population, new GenomeComparator(maximize));
		for (int i = 0; i < population.size(); i++) {
			population.get(i).setFitnessRank(i + 1);
		}
	}

	// only after elite is in nextGeneration
	public void assignDeviation() {
		for (Genome<T> g : population) {
			g.setDeviation(nextGeneration);
		}
	}

	public void assignDeviationRank() {
		Collections.sort(population, new GenomeComparator(maximize,
				GenomeComparator.DEVIATION));
		for (int i = 0; i < population.size(); i++) {
			population.get(i).setDeviationRank(i + 1);
		}
	}

	public void sortByCombineRank() {
		assignFitnessRank();
		assignDeviation();
		assignDeviationRank();
		Collections.sort(population, new GenomeComparator(maximize,
				GenomeComparator.COMBINE));
	}

	public void adaptMutationRate() {
		if (generationCount > 0
				&& generationCount % (generationSize * ADAPTIVE_MUTATION_RATE) == 0)
			this.MUTATION_RATE /= 2;
	}

	public void selectElite() {
		nextGeneration = new ArrayList<Genome<T>>();
		tempPopulation = new ArrayList<Genome<T>>();
		tempPopulation.addAll(population);
		for (int j = 0; j < Math.ceil(populationSize * ELITE); j++) {
			nextGeneration.add(Collections.min(tempPopulation,
					new GenomeComparator(maximize)).clone());
			tempPopulation.remove(Collections.min(tempPopulation,
					new GenomeComparator(maximize)));
			activeIndex = getAdjustedActiveIndex(1);
		}
	}

	public double getGlobalDiversity() {
		return getDiversity(population, population);
	}

	public void initLocalDiversity() {
		localDiversity = new double[populationSizeX][populationSizeY];
		double range = updateLocalDiversity(false);
		output.initializeLocalDiversity(localDiversity, range);
	}

	public double updateLocalDiversity(boolean displayUpdate) {
		double max = 0, min = Double.MAX_VALUE;
		for (int i = 0; i < localDiversity.length; i++)
			for (int j = 0; j < localDiversity[0].length; j++) {
				localDiversity[i][j] = calculateLocalDiversity(i, j);
				if (localDiversity[i][j] > max)
					max = localDiversity[i][j];
				if (localDiversity[i][j] < min)
					min = localDiversity[i][j];
			}
		if (displayUpdate) {
			output.setRange(max);
			for (int i = 0; i < localDiversity.length; i++)
				for (int j = 0; j < localDiversity[0].length; j++)
					output.setLocalDiversity(i, j, localDiversity[i][j]);
			output.repaint();
		}
		return max;
	}

	public double calculateLocalDiversity(int x, int y) {
		ArrayList<Genome<T>> centerGenome = new ArrayList<Genome<T>>(), surroundingGenome = new ArrayList<Genome<T>>();
		centerGenome.add(population2D.get(x).get(y));
		surroundingGenome.add(population2D.get(
				getAdjustedIndex(x, 1, populationSizeX)).get(y));
		surroundingGenome.add(population2D.get(
				getAdjustedIndex(x, -1, populationSizeX)).get(y));
		surroundingGenome.add(population2D.get(x).get(
				getAdjustedIndex(y, 1, populationSizeY)));
		surroundingGenome.add(population2D.get(x).get(
				getAdjustedIndex(y, -1, populationSizeY)));
		return getDiversity(centerGenome, surroundingGenome);
	}

	public Genome<T> localSearch(Genome<T> g) {
		return tabuSearch(g);
	}

	public Genome<T> tabuSearch(Genome<T> g) {
		return g.clone();
	}

	public void start() {
		t = new Thread(this);
		t.start();
	}

	public void stop() {
		t = null;
	}

	public void run() {
		this.optimize();
	}

	public synchronized void optimize() {
		initPopulation();
		initLocalDiversity();
		Thread thisThread = Thread.currentThread();
		generationCount = 0;
		evaluate();
		showResult(generationCount);
		waitForNextStep();
		for (; generationCount < generationSize && !reachGlobalOptimum(); generationCount++) {
			if (t != thisThread)
				break;
			if (ADAPTIVE_MUTATION) {
				adaptMutationRate();
			}
			int counter = 0;
			activeIndex = 0;

			resetMatingCount();

			selectElite();
			if (STEP_DETAIL)
				updateActiveIndex(activeIndex);
			while (nextGeneration.size() < populationSize
					&& (!COUNTER_LOCK || counter < populationSize)) {
				select();
				counter++;
			}

			setMatingCount();

			if (SHOW_MATING_PROB || PRINT_MATING_PROB)
				measureMatingProbability();

			// specialShowMatingCount();

			population = nextGeneration;
			population2D = PopulationFactory.get2DPopulation(population,
					populationSizeX);

			setSuccessMatingCount();

			evaluate();
			showResult(generationCount + 1);
			waitForNextStep();
		}
		if (generationCount == generationSize || reachGlobalOptimum()) {
			if (reachGlobalOptimum())
				System.out.println(bestGenome.printChromosome());
			generationUsed = generationCount;
			if (LOG_RESULT)
				printSummary();
			System.out.println("Finish");
		} else
			System.out.println("Abort");
		synchronized (GUI.gaList) {
			GUI.gaList.remove(this);
			GUI.gaList.notifyAll();
		}
	}

	private void resetMatingCount() {
		matingCount00 = 0;
		matingCount01 = 0;
		matingCount11 = 0;
		successMatingCount00 = 0;
		successMatingCount01 = 0;
		successMatingCount11 = 0;
	}

	private void setMatingCount() {
		arrayMatingCount00[generationCount] = matingCount00;
		arrayMatingCount01[generationCount] = matingCount01;
		arrayMatingCount11[generationCount] = matingCount11;
	}

	private void setSuccessMatingCount() {
		arraySuccessMatingCount00[generationCount] = successMatingCount00;
		arraySuccessMatingCount01[generationCount] = successMatingCount01;
		arraySuccessMatingCount11[generationCount] = successMatingCount11;
	}

	public void setBestAndNonBestList() {
		nonBestList = new ArrayList<Genome<T>>();
		bestList = new ArrayList<Genome<T>>();
		for (Genome<T> g : population) {
			if (!isBestSoFar(g))
				nonBestList.add(g);
			else
				bestList.add(g);
		}
	}

	public void evaluate() {
		double best, sum = 0, mean;
		int bestCount = 0;
		boolean moreExploit = false, moreExplore = false;
		if (maximize)
			best = Double.MIN_VALUE;
		else
			best = Double.MAX_VALUE;
		for (Genome<T> g : population) {
			double gFitness = g.getFitnessValue();
			if ((maximize && gFitness > best) || (!maximize && gFitness < best)) {
				best = gFitness;
				bestGenome = g;
			}
			sum += gFitness;
		}
		for (Genome<T> g : population)
			if (g.getFitnessValue() == best)
				bestCount++;

		arrayBestCount[generationCount] = bestCount;

		if ((maximize && best > BEST_SO_FAR)
				|| (!maximize && best < BEST_SO_FAR))
			BEST_SO_FAR = best;

		mean = sum / population.size();
		if (DIFF_MEAN_FITNESS >= 0) {
			if (mean - MEAN_FITNESS < (1 - ADAPTIVE_FACTOR) * DIFF_MEAN_FITNESS)
				moreExploit = true;
			if (mean - MEAN_FITNESS > (1 + ADAPTIVE_FACTOR) * DIFF_MEAN_FITNESS)
				moreExplore = true;
		}

		updateLocalDiversity(true);

		if (SHOW_DIVERSITY) {
			GLOBAL_DIVERSITY = getGlobalDiversity();
		}

		DIFF_MEAN_FITNESS = mean - MEAN_FITNESS;
		MEAN_FITNESS = mean;

		if (ADAPTIVE_PARAMETER) {
			if (STRUCTURE == CELLULAR1D) {
				if (moreExploit && 2 * radius + 1 < populationSize)
					radius++;
				else if (moreExplore && 2 * radius + 1 > POOL_SIZE)
					radius--;
			} else if (STRUCTURE == CELLULAR2D)
				if (SELECTION_SCHEME == POLYNOMIAL_SELECTION) {
					POLYNOMIAL_PARAMETER = findBestParameter(population2D,
							bestCount);
					//setSelectionProbability2D();
				} else if (SELECTION_SCHEME == CENTRIC_SELECTION) {
					CENTRIC_PARAMETER = findBestParameter(population2D,
							bestCount);
					setSelectionProbability2D();
				}
		}
	}

	public boolean reachGlobalOptimum() {
		return false;
	}

	public void showResult(int iteration) {
		boolean printBestGenome = false, printAllGenome = false;
		if (printAllGenome)
			for (Genome<T> g : population) {
				System.out.println(g.printChromosome());
			}
		if (printBestGenome)
			System.out.println("Best : " + bestGenome.printChromosome());
		if (SHOW_MATING_PROB) {
			if (iteration > 0) {
				double p00 = 1.0 * successMatingCount00 / matingCount00, p01 = 1.0
						* successMatingCount01 / matingCount01, p11 = 1.0
						* successMatingCount11 / matingCount11;

				if (p00 > 0)
					output.updateGraphValue(1, 0, iteration, p00);
				if (p01 > 0)
					output.updateGraphValue(1, 1, iteration, p01);
				if (p11 > 0)
					output.updateGraphValue(1, 2, iteration, p11);
			}
		} else if (SHOW_MATING_COUNT) {
			double n00 = estimatedMatingCount(population2D,
					selectionProbability2D, type00), n01 = estimatedMatingCount(
					population2D, selectionProbability2D, type01), n11 = estimatedMatingCount(
					population2D, selectionProbability2D, type11);
			if (n00 > 0)
				output.updateGraphValue(1, 0, iteration, n00);
			if (n01 > 0)
				output.updateGraphValue(1, 1, iteration, n01);
			if (n11 > 0)
				output.updateGraphValue(1, 2, iteration, n11);
			if (PRINT_MATING_COUNT) {
				System.out.println(matingCount00 + " " + n00);
				System.out.println(matingCount01 + " " + n01);
				System.out.println(matingCount11 + " " + n11);
			}
		} else {
			output.updateGraphValue(0, 0, iteration, BEST_SO_FAR);
			output.updateGraphValue(0, 1, iteration, MEAN_FITNESS);
			if (SHOW_DIVERSITY && iteration < generationSize) {
				arrayGlobalDiversity[iteration] = GLOBAL_DIVERSITY;
				output.updateGraphValue(1, 0, iteration, GLOBAL_DIVERSITY);
			}
		}
		if (SHOW_OPTIMAL_PARAMETER)
			if (SELECTION_SCHEME == POLYNOMIAL_SELECTION)
				output.updateGraphValue(1, 0, iteration, POLYNOMIAL_PARAMETER);
			else if (SELECTION_SCHEME == CENTRIC_SELECTION)
				output.updateGraphValue(1, 0, iteration, POLYNOMIAL_PARAMETER);
		output.updateAlgorithmStatus(BEST_SO_FAR, evaluationCount);
		output.updateGraphImage();
	}

	public void printSummary() {
		if (PRINT_MATING_PROB)
			output.excelHandler.printVerticleResults(
					(GeneticAlgorithm<?>) this, ExcelHandler.MATING_PROB);
		// output.excelHandler.printOnlyResults((GeneticAlgorithm<?>) this);
		else
			output.excelHandler.printResults((GeneticAlgorithm<?>) this,
					ExcelHandler.NORMAL, PRINT_GRAPH);
	}

	public void waitForNextStep() {
		if (t != null) {
			synchronized (this.t) {
				while (stepNum < nextStepNum && STEP)
					try {
						t.wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
			}
			nextStepNum = stepNum + 1;
		}
	}

	public void pause(int time) {
		try {
			Thread.sleep(time);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public void updateIndividualsOnDisplay(Genome<T> g1, Genome<T> g2, int step) {

	}

	public void updateActiveIndex(int activeIndex) {

	}

	public int measureMatingProbability() {
		setBestAndNonBestList();
		Genome<T> g1 = null, g2 = null;
		if (nonBestList.size() > 0) {
			while (matingCount00 < matingSampleSize) {
				g1 = nonBestList.get((int) (random.nextDouble() * nonBestList
						.size()));
				g2 = nonBestList.get((int) (random.nextDouble() * nonBestList
						.size()));
				mate(g1, g2);
			}
			while (matingCount01 < matingSampleSize) {
				g1 = nonBestList.get((int) (random.nextDouble() * nonBestList
						.size()));
				g2 = bestList
						.get((int) (random.nextDouble() * bestList.size()));
				mate(g1, g2);
			}
		}
		while (matingCount11 < matingSampleSize) {
			g1 = bestList.get((int) (random.nextDouble() * bestList.size()));
			g2 = bestList.get((int) (random.nextDouble() * bestList.size()));
			mate(g1, g2);
		}
		return bestList.size();
	}

	public double estimatedMatingCount(
			ArrayList<ArrayList<Genome<T>>> population2D,
			double[][] selectProb2D, int type) {
		double expectedNum = 0;
		for (int i = 0; i < populationSizeX; i++)
			for (int j = 0; j < populationSizeY; j++) {
				double prob0 = 0;
				for (int k = 0; k < selectProb2D.length; k++)
					for (int l = 0; l < selectProb2D[0].length; l++) {
						prob0 += (1 - isBestSoFarFunction(population2D.get(
								(i + k) % populationSizeX).get(
								(j + l) % populationSizeY)))
								* selectProb2D[k][l];
					}
				double prob0Win = Math.pow(prob0, POOL_SIZE), prob1Win = 1 - prob0Win;
				if (type == type00)
					expectedNum += prob0Win * prob0Win;
				else if (type == type01)
					expectedNum += 2 * prob0Win * prob1Win;
				else if (type == type11)
					expectedNum += prob1Win * prob1Win;
			}
		return expectedNum;
	}

	public Genome<T> findBestIndividual(
			ArrayList<ArrayList<Genome<T>>> population2D) {
		Genome<T> bestIndividual = null;
		double best;
		if (maximize)
			best = Double.MIN_VALUE;
		else
			best = Double.MAX_VALUE;
		for (ArrayList<Genome<T>> gArray : population2D) {
			for (Genome<T> g : gArray) {
				double gFitness = g.getFitnessValue();
				if ((maximize && gFitness > best)
						|| (!maximize && gFitness < best)) {
					best = gFitness;
					bestIndividual = g;
				}
			}
		}
		return bestIndividual;
	}

	public int countBestIndividual(ArrayList<ArrayList<Genome<T>>> population2D) {
		int bestCount = 0;
		Genome<T> bestIndividual = findBestIndividual(population2D);
		for (ArrayList<Genome<T>> gArray : population2D) {
			for (Genome<T> g : gArray) {
				if (g.getFitnessValue() == bestIndividual.getFitnessValue())
					bestCount++;
			}
		}
		return bestCount;
	}

	public int countBestSoFarIndividual(
			ArrayList<ArrayList<Genome<T>>> population2D) {
		int bestCount = 0;
		for (ArrayList<Genome<T>> gArray : population2D) {
			for (Genome<T> g : gArray) {
				if (isBestSoFar(g))
					bestCount++;
			}
		}
		return bestCount;
	}

	public double findBestParameter(
			ArrayList<ArrayList<Genome<T>>> population2D, int bestCount) {
		double bestParameter = 0, functionValue, previousFunctionValue, bestFunctionValue, p00, p01, p11, n00, n01, n11;
		double epsilon = 1E-10, stepSize = 0.01, acc = 1.2;
		double[] accArray = { acc, 1.0 / acc, 0.0, -1.0 / acc, -acc };
		double[][] selectProb2D;
		boolean printPEMgraph = true;
		int bestAcc = 0;
		if (SELECTION_SCHEME == POLYNOMIAL_SELECTION)
			bestParameter = 0;
		else if (SELECTION_SCHEME == CENTRIC_SELECTION)
			bestParameter = 0.2;
		population2D = PopulationFactory.get2DPopulationClone(population2D);
		p00 = matingProbability.getMatingProb(bestCount, type00);
		p01 = matingProbability.getMatingProb(bestCount, type01);
		p11 = matingProbability.getMatingProb(bestCount, type11);
		functionValue = Double.MIN_VALUE;
		bestFunctionValue = Double.MIN_VALUE;
		if (printPEMgraph && bestCount > 1) {
			System.out.println("Best Count " + bestCount);
			selectProb2D = initializeSelectionProbability2D();
			for (double i = 0; i <= 10; i += 0.5) {
				setSelectionProbability2D(selectProb2D, i);
				n00 = estimatedMatingCount(population2D, selectProb2D, type00);
				n01 = estimatedMatingCount(population2D, selectProb2D, type01);
				n11 = estimatedMatingCount(population2D, selectProb2D, type11);
				functionValue = estimatedNewBestProbability(p00, p01, p11, n00,
						n01, n11);
				System.out.println(i + " " + functionValue);
			}
		}
		do {
			previousFunctionValue = bestFunctionValue;
			bestFunctionValue = Double.MIN_VALUE;
			selectProb2D = initializeSelectionProbability2D();
			for (int i = 0; i < accArray.length; i++) {
				double temp = bestParameter + stepSize * accArray[i];
				if (SELECTION_SCHEME == POLYNOMIAL_SELECTION && temp < 0
						|| SELECTION_SCHEME == CENTRIC_SELECTION && temp < 0.2)
					continue;
				setSelectionProbability2D(selectProb2D, temp);
				n00 = estimatedMatingCount(population2D, selectProb2D, type00);
				n01 = estimatedMatingCount(population2D, selectProb2D, type01);
				n11 = estimatedMatingCount(population2D, selectProb2D, type11);
				functionValue = estimatedNewBestProbability(p00, p01, p11, n00,
						n01, n11);
				if (functionValue > bestFunctionValue) {
					bestFunctionValue = functionValue;
					bestAcc = i;
				}
				if (PRINT_OPTIMAL_PARAMETER
						&& bestCount % bestIndividualInterval == 0
						&& bestCount < populationSize)
					System.out.println(i + " " + temp + " " + functionValue
							+ " " + n00 + " " + n01 + " " + n11);
			}
			if (accArray[bestAcc] != 0) {
				bestParameter += stepSize * accArray[bestAcc];
				stepSize *= accArray[bestAcc];
			}
			if (PRINT_OPTIMAL_PARAMETER
					&& bestCount % bestIndividualInterval == 0
					&& bestCount < populationSize)
				System.out.println("iteration: " + bestCount + ", "
						+ bestParameter);
		} while (Math.abs(bestFunctionValue - previousFunctionValue) > epsilon);
		if (PRINT_OPTIMAL_PARAMETER && bestCount < populationSize)
			System.out.println("generation: " + bestCount + ", "
					+ bestParameter);
		return bestParameter;
	}

	public void specialShowMatingCount() {
		setBestAndNonBestList();
		if (bestList.size() % bestIndividualInterval == 0
				&& bestList.size() < populationSize) {
			double p00 = matingProbability.getMatingProb(bestList.size(),
					type00);
			double p01 = matingProbability.getMatingProb(bestList.size(),
					type01);
			double p11 = matingProbability.getMatingProb(bestList.size(),
					type11);
			for (int param = 0; param < 10; param++) {
				double[][] selectProb2D = initializeSelectionProbability2D();
				setSelectionProbability2D(selectProb2D, param);
				double n00 = estimatedMatingCount(population2D, selectProb2D,
						type00);
				double n01 = estimatedMatingCount(population2D, selectProb2D,
						type01);
				double n11 = estimatedMatingCount(population2D, selectProb2D,
						type11);
				double functionValue = estimatedNewBestProbability(p00, p01,
						p11, n00, n01, n11);
				System.out.println(bestList.size() + " " + param + " "
						+ functionValue + " " + n00 + " " + n01 + " " + n11);
			}
		}
	}

	public static double estimatedNewBestProbability(double p00, double p01,
			double p11, double n00, double n01, double n11) {
		return 1 - Math.pow(1 - p00, n00) * Math.pow(1 - p01, n01)
				* Math.pow(1 - p11, n11);
	}
}

class TestSelectivePressure extends BitStringGeneticAlgorithm {

	public double[][] bestIndividualProb, nextGenerationProb;

	public static boolean showProb = false;

	public TestSelectivePressure(Display output) {
		super(output, 1);
		initPopulation();
	}

	public BitStringTestFunction bsb;

	public int takeOverTime = -1;

	private double smallDifference = 0.1;

	public void initDefaultParameter() {
		CHROMOSOME_SIZE = 1;
		POPULATION_SIZE = 4096;
		GENERATION_SIZE = 2000;
		BEST_SO_FAR = Double.MAX_VALUE;
		maximize = false;
		generationCount = 0;
	}

	public void initPopulation() {
		bf = new BitStringTestFunction();
		bf.initialize(testNumber, this);
		BitStringGenome bad = new BitStringGenome(chromosomeSize, bf, this), good = new BitStringGenome(
				chromosomeSize, bf, this);
		for (int i = 0; i < bad.chromosome.length; i++)
			bad.chromosome[i] = 0;
		for (int i = 0; i < good.chromosome.length; i++)
			good.chromosome[i] = 1;
		for (int i = 0; i < populationSize; i++)
			population.add(bad.clone());
		int initialBestIndex = (int) (populationSizeX / 2 * populationSizeX + populationSizeX / 2);
		population.set(initialBestIndex, good);
		population2D = PopulationFactory.get2DPopulation(population,
				populationSizeX);
		bestIndividualProb = new double[populationSizeX][populationSizeY];
		for (int i = 0; i < bestIndividualProb.length; i++)
			for (int j = 0; j < bestIndividualProb[0].length; j++)
				bestIndividualProb[i][j] = 1 - population2D.get(i).get(j)
						.getFitnessValue();
		output.initialTestSelectivePressure(bestIndividualProb, chromosomeSize);
		arrayBestCount = new int[generationSize];
	}

	public synchronized void optimize() {
		Thread thisThread = Thread.currentThread();
		optimizingTime = System.currentTimeMillis();
		for (; generationCount < generationSize && !reachGlobalOptimum(); generationCount++) {
			if (t != thisThread || takeOverTime != -1)
				break;
			if (!showProb) {
				activeIndex = 0;
				nextGeneration = new ArrayList<Genome<int[]>>();
				while (nextGeneration.size() < populationSize) {
					GenomeComparator gc = new GenomeComparator(maximize);
					Genome<int[]> g = getTournamentWinner();
					if (gc.compare(g, population.get(activeIndex)) <= 0) {
						nextGeneration.add(g);
						activeIndex = getAdjustedActiveIndex(1);
					}
				}
				population = nextGeneration;
				population2D = PopulationFactory.get2DPopulation(population,
						populationSizeX);
			} else {
				nextGenerationProb = new double[populationSizeX][populationSizeY];
				for (int j = 0; j < nextGenerationProb.length; j++)
					for (int k = 0; k < nextGenerationProb[0].length; k++) {
						double sum = 0;
						for (int l = 0; l < selectionProbability2D.length; l++)
							for (int m = 0; m < selectionProbability2D[0].length; m++) {
								int index1 = (j - selectionProbability2D.length
										/ 2 + l + nextGenerationProb.length)
										% nextGenerationProb.length;
								int index2 = (k
										- selectionProbability2D[0].length / 2
										+ m + nextGenerationProb[0].length)
										% nextGenerationProb[0].length;
								if (!(index1 == j && index2 == k))
									sum += bestIndividualProb[index1][index2]
											* selectionProbability2D[l][m];
							}
						nextGenerationProb[j][k] = bestIndividualProb[j][k]
								+ (1 - bestIndividualProb[j][k]) * sum;
					}
				bestIndividualProb = nextGenerationProb;
			}
			evaluate();
			showResult(generationCount);
			waitForNextStep();
		}
		if (generationCount == generationSize || reachGlobalOptimum()) {
			if (LOG_RESULT)
				printSummary();
		}
		synchronized (GUI.gaList) {
			GUI.gaList.remove(this);
			GUI.gaList.notifyAll();
		}
	}

	public void evaluate() {
		double best;
		int bestCount = 0;
		if (maximize)
			best = Double.MIN_VALUE;
		else
			best = Double.MAX_VALUE;
		if (!showProb) {
			if (STRUCTURE == CELLULAR2D) {
				for (int i = 0; i < populationSizeX; i++)
					for (int j = 0; j < populationSizeY; j++) {
						Genome<int[]> g = population2D.get(i).get(j);
						double gFitness = g.getFitnessValue();
						if ((maximize && gFitness > best)
								|| (!maximize && gFitness < best)) {
							best = gFitness;
							bestGenome = g;
						}
					}
				for (int i = 0; i < populationSizeX; i++)
					for (int j = 0; j < populationSizeY; j++) {
						Genome<int[]> g = population2D.get(i).get(j);
						if (g.getFitnessValue() == best) {
							output.setBestIndividual(i, j, 1.0 - g
									.getFitnessValue());
							bestCount++;
						}
					}
			} else {
				for (Genome<int[]> g : population) {
					double gFitness = g.getFitnessValue();
					if ((maximize && gFitness > best)
							|| (!maximize && gFitness < best)) {
						best = gFitness;
						bestGenome = g;
					}
				}
				for (Genome<int[]> g : population)
					if (g.getFitnessValue() == best)
						bestCount++;
			}
			arrayBestCount[generationCount] = bestCount;
			if (takeOverTime == -1 && bestCount == populationSize)
				takeOverTime = generationCount;
		} else {
			boolean takeOver = true;
			for (int i = 0; i < bestIndividualProb.length; i++)
				for (int j = 0; j < bestIndividualProb[0].length; j++) {
					output.setBestIndividual(i, j, bestIndividualProb[i][j]);
					if (1.0 - bestIndividualProb[i][j] > smallDifference) {
						takeOver = false;
					}
				}
			if (takeOver)
				takeOverTime = generationCount;
		}
	}

	public void showResult(int n) {
		output.updateAlgorithmStatus(n);
		output.repaint();
		// pause(100);
	}

	public void printSummary() {
		output.excelHandler.printResults((GeneticAlgorithm<?>) this,
				ExcelHandler.SELECTIVE_PRESSURE, PRINT_GRAPH);
		System.out.println("Select Parent Count: " + selectParentCount);
		System.out.println("Select Parent Time: " + 1.0 * selectParentTime
				/ selectParentCount);
		System.out.println("Create Neighborhood Count: "
				+ createNeighborhoodCount);
		System.out.println("Create Neighborhood Time: " + 1.0
				* createNeighborhoodTime / createNeighborhoodCount);
		optimizingTime = System.currentTimeMillis() - optimizingTime;
		System.out.println("Optimizing Time: " + optimizingTime);
		System.out.println("Different: "
				+ (optimizingTime - createNeighborhoodTime));
	}

	public boolean reachGlobalOptimum() {
		if (takeOverTime != -1)
			return true;
		else
			return false;
	}
}

class BitStringGeneticAlgorithm extends GeneticAlgorithm<int[]> {

	protected int testNumber;

	public BitStringTestFunction bf;

	public BitStringGeneticAlgorithm(Display output, int testNumber) {
		super(output);
		this.output.initXYGraph();
		this.testNumber = testNumber;
	}

	public void initDefaultParameter() {
		CHROMOSOME_SIZE = 48;
		POPULATION_SIZE = 400;
		GENERATION_SIZE = 2000;
		BEST_SO_FAR = Double.MAX_VALUE;
		MUTATION_RATE = 1.0 / CHROMOSOME_SIZE;
		maximize = false;
	}

	public void initPopulation() {
		bf = new BitStringTestFunction();
		bf.initialize(testNumber, this);
		population = PopulationFactory.get1DPopulation(new BitStringGenome(
				chromosomeSize, bf, this), populationSize);
		population2D = PopulationFactory.get2DPopulation(population,
				populationSizeX);
	}

	public boolean reachGlobalOptimum() {
		if (BEST_SO_FAR == bf.optimum)
			return true;
		else
			return false;
	}

	public double getDiversity(ArrayList<Genome<int[]>> a,
			ArrayList<Genome<int[]>> b) {
		double diversity = 0;
		for (Genome<int[]> g1 : a) {
			for (Genome<int[]> g2 : b) {
				diversity += BitStringTestFunction.hammingDistance(
						g1.chromosome, g2.chromosome)
						/ (double) chromosomeSize;
			}
		}
		return diversity / (a.size() * b.size());
	}
}

class PermutationGeneticAlgorithm extends BitStringGeneticAlgorithm {

	public PermutationGeneticAlgorithm(Display output, int testNumber) {
		super(output, testNumber);
		matingProbability = new MatingProbability("nug30");
	}

	public void initPopulation() {
		bf = new BitStringTestFunction();
		bf.initialize(testNumber, this);
		population = PopulationFactory.get1DPopulation(new PermutationGenome(
				chromosomeSize, bf, this), populationSize);
		population2D = PopulationFactory.get2DPopulation(population,
				populationSizeX);
	}
}

class BitStringTestFunction {

	private static double[] MMDP_functionValue = { 1.000000, 0.000000,
			0.360384, 0.640576, 0.360384, 0.000000, 1.000000 };

	private static String instanceQAP = "nug30";

	public int testNumber;

	public double optimum = 0;

	private Random random;

	private int randomSum, randomSumScale = 10, peakNumber = 100, n, z, b, l;

	private double expectedNumberSumRatio = 0.5;

	private int[] randomNumber;

	private int[][] peakGenome, A, B;

	private long DEFAULT_SEED = 0;

	private BitStringGeneticAlgorithm bsga;

	public void initialize(int testNumber, BitStringGeneticAlgorithm bsga) {

		this.testNumber = testNumber;
		this.bsga = bsga;
		this.random = new Random(DEFAULT_SEED);
		switch (this.testNumber) {
		case 1:
			this.bsga.PROBLEM_NAME = "ONEMAX";
			randomSum = this.bsga.chromosomeSize;
			randomNumber = new int[this.bsga.chromosomeSize];
			for (int i = 0; i < this.bsga.chromosomeSize; i++)
				randomNumber[i] = 1;
			this.bsga.maximize = false;
			this.bsga.BEST_SO_FAR = Double.MAX_VALUE;
			optimum = 0;
			break;
		case 2:
			this.bsga.PROBLEM_NAME = "SUBSETSUM";
			randomSum = (int) (random.nextDouble() * randomSumScale * this.bsga.chromosomeSize);
			randomNumber = new int[this.bsga.chromosomeSize];
			for (int i = 0; i < this.bsga.chromosomeSize; i++) {
				randomNumber[i] = (int) (random.nextDouble() * (randomSumScale / expectedNumberSumRatio));
			}
			this.bsga.maximize = false;
			this.bsga.BEST_SO_FAR = Double.MAX_VALUE;
			optimum = 0;
			break;
		case 3:
			this.bsga.PROBLEM_NAME = "MMDP";
			this.bsga.maximize = true;
			this.bsga.BEST_SO_FAR = Double.MIN_VALUE;
			optimum = this.bsga.chromosomeSize / 6;
			break;
		case 4:
			this.bsga.PROBLEM_NAME = "P-PEAKS";
			this.bsga.maximize = true;
			this.bsga.BEST_SO_FAR = Double.MIN_VALUE;
			optimum = 1.0;
			peakGenome = new int[peakNumber][this.bsga.chromosomeSize];
			for (int i = 0; i < peakNumber; i++) {
				peakGenome[i] = new int[this.bsga.chromosomeSize];
				for (int j = 0; j < this.bsga.chromosomeSize; j++) {
					peakGenome[i][j] = (int) Math.round(random.nextDouble());
				}
			}
			break;
		case 5:
			this.bsga.PROBLEM_NAME = "COUNTSAT";
			this.bsga.maximize = true;
			this.bsga.BEST_SO_FAR = Double.MIN_VALUE;
			this.bsga.MUTATION_RATE = 1.0 / (2 * this.bsga.chromosomeSize);
			optimum = countSat(bsga.chromosomeSize, bsga.chromosomeSize);
			break;
		case 6:
			this.bsga.PROBLEM_NAME = "QAP_" + instanceQAP;
			this.bsga.maximize = false;
			this.bsga.BEST_SO_FAR = Double.MAX_VALUE;
			this.bsga.MUTATION_RATE = 1.0;
			this.bsga.CROSSOVER_RATE = 1.0;

			File file = new File("." + GUI.fs + "QAP" + GUI.fs + instanceQAP
					+ ".dat");
			FileInputStream in = null;
			try {
				in = new FileInputStream(file);
				BufferedReader reader = new BufferedReader(
						new InputStreamReader(in));
				String line = null;
				line = reader.readLine();
				line = line.trim();
				n = Integer.parseInt(line);
				bsga.setParameter(n, bsga.generationSize, bsga.populationSize);
				if (n == 30)
					optimum = 6124;
				A = new int[n][n];
				B = new int[n][n];
				line = reader.readLine();
				int i = 0, k = 0, l = 0, temp = 0;
				while ((line = reader.readLine()) != null) {
					if (i == n) {
						i = 0;
						k++;
						continue;
					}
					String[] s = line.split("\\s+");
					for (int j = 0; j < s.length; j++) {
						try {
							temp = Integer.parseInt(s[j]);
						} catch (NumberFormatException e) {
							continue;
						}
						if (k == 0)
							A[i][l] = temp;
						else
							B[i][l] = temp;
						l++;
					}
					if (l == n) {
						i++;
						l = 0;
					}

				}
			} catch (IOException x) {
				System.err.println(x);
			} finally {
				if (in != null)
					try {
						in.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
			}
			break;
		case 7:
			this.bsga.PROBLEM_NAME = "LOLZ";
			this.bsga.maximize = true;
			this.bsga.BEST_SO_FAR = Double.MIN_VALUE;
			this.bsga.MUTATION_RATE = 1.0 / this.bsga.chromosomeSize;
			this.b = (int) Math.sqrt(this.bsga.chromosomeSize);
			this.l = this.bsga.chromosomeSize / b;
			this.z = this.l - 1;
			optimum = this.bsga.chromosomeSize;
			break;
		}
	}

	public double getFunctionValue(int[] x) {
		double sum = 0;
		int subSum = 0, result = 0;
		switch (testNumber) {
		case 3:
			for (int i = 0; i < x.length / 6; i++) {
				subSum = 0;
				for (int j = i * 6; j < i * 6 + 6; j++) {
					subSum += x[j];
				}
				sum += MMDP_functionValue[subSum];
			}
			return sum;
		case 4:
			double max = 0.0;
			for (int i = 0; i < peakNumber; i++) {
				double temp = this.bsga.chromosomeSize
						- hammingDistance(x, peakGenome[i]);
				if (temp > max)
					max = temp;
			}
			return max / this.bsga.chromosomeSize;
		case 5:
			result = countSat(countNoOfOnes(x), bsga.chromosomeSize);
			return result;
		case 6:
			for (int i = 0; i < n; i++)
				for (int j = 0; j < n; j++)
					result += A[i][j] * B[x[i]][x[j]];
			return result;
		case 7:
			for (int i = 0; i < b; i++) {
				subSum = 1;
				for (int j = 0; j < i * l; j++) {
					int[] block = new int[l];
					System.arraycopy(x, i * l, block, 0, l);
					subSum *= x[j]
							* (leading(block, true) + Math.min(z, leading(
									block, false)));
				}
				result += subSum;
			}
			return result - 1;

		default:
			for (int i = 0; i < x.length; i++) {
				if (x[i] == 1)
					sum += randomNumber[i];
			}
			return Math.abs(randomSum - sum);
		}
	}

	public static int hammingDistance(int[] a, int[] b) {
		int lengthDifferent = a.length - b.length;
		int shorterLength;
		int sum = 0;
		if (lengthDifferent > 0)
			shorterLength = a.length;
		else
			shorterLength = b.length;
		sum += Math.abs(lengthDifferent);
		for (int i = 0; i < shorterLength; i++) {
			if (a[i] != b[i])
				sum += 1;
		}
		return sum;
	}

	public static int combination(int n, int r) {
		int result = n;
		for (int i = 1; i < r; i++) {
			result *= n - i;
		}
		for (int i = 0; i < r; i++) {
			result /= r - i;
		}
		return result;
	}

	public static int countSat(int s, int n) {
		int result = s + n * (n - 1) * (n - 2) - 2 * (n - 2)
				* combination(s, 2) + 6 * combination(s, 3);
		return result;
	}

	public static int countNoOfOnes(int[] x) {
		int sum = 0;
		for (int i = 0; i < x.length; i++) {
			if (x[i] == 1)
				sum++;
		}
		return sum;
	}

	public static int leading(int[] x, boolean isLeadingOnes) {
		for (int i = 0; i < x.length; i++) {
			if (isLeadingOnes && x[i] == 0 || !isLeadingOnes && x[i] == 1)
				return i;
		}
		return x.length;
	}

	public static double convertParamC2P(double beta) {
		return Math.log((4 * beta) / (1 - beta)) / Math.log(2);
	}
}

class GridGeneticAlgorithm extends BitStringGeneticAlgorithm {

	public int[][] FITNESS;

	public int LOWER_BOUND = 0;

	public int UPPER_BOUND = 20;

	public double CENTRAL_AREA_PERCENTAGE = 0.2;

	public double EDGE_AREA_PERCENTAGE = 0.15;

	public boolean randomInitial = true, valley = true;

	public GridGeneticAlgorithm(Display output) {
		super(output, 0);
		initializeFitness();
		this.output.initialiseGridData(FITNESS, UPPER_BOUND - LOWER_BOUND + 1);
	}

	public void initDefaultParameter() {
		CHROMOSOME_SIZE = 2;
		POPULATION_SIZE = 25;
		GENERATION_SIZE = 50;
		MUTATION_RATE = 1.0 / CHROMOSOME_SIZE;
		maximize = true;
	}

	public void initPopulation() {
		PROBLEM_NAME = "VALLEY";
		output.initialisePopulation(populationSize);
		population = PopulationFactory.get1DPopulation(new GridGenome(
				chromosomeSize, this), populationSize);
		population2D = PopulationFactory.get2DPopulation(population,
				populationSizeX);
		bestGenome = (GridGenome) population.get(0);
	}

	public void initializeFitness() {
		constantFitness();
		// randomFitness();
		if (valley) {
			int centerArea = (int) Math.round(UPPER_BOUND
					* CENTRAL_AREA_PERCENTAGE), edgeArea = (int) Math
					.round(UPPER_BOUND * EDGE_AREA_PERCENTAGE);
			int lowBox = (int) (UPPER_BOUND / 2.0 - centerArea), highBox = (int) Math
					.ceil(UPPER_BOUND / 2.0 + centerArea);
			for (int i = edgeArea; i <= UPPER_BOUND - edgeArea; i++)
				for (int j = edgeArea; j <= UPPER_BOUND - edgeArea; j++)
					if (i < lowBox || i > highBox || j < lowBox || j > highBox)
						FITNESS[i][j] = 0;
		}
	}

	public void randomFitness() {
		FITNESS = new int[UPPER_BOUND + 1][UPPER_BOUND + 1];
		for (int i = 0; i < FITNESS.length; i++)
			for (int j = 0; j < FITNESS[0].length; j++) {
				FITNESS[i][j] = (int) (random.nextDouble() * 100 / (Math
						.abs(UPPER_BOUND / 2 - i)
						+ Math.abs(UPPER_BOUND / 2 - j) + 1));
			}
	}

	public void constantFitness() {
		FITNESS = new int[UPPER_BOUND + 1][UPPER_BOUND + 1];
		for (int i = 0; i < FITNESS.length; i++)
			for (int j = 0; j < FITNESS[0].length; j++) {
				int x, y;
				if (i < FITNESS.length / 2)
					x = i;
				else
					x = FITNESS.length - 1 - i;
				if (j < FITNESS[0].length / 2)
					y = j;
				else
					y = FITNESS[0].length - 1 - j;
				FITNESS[i][j] = x + y + 1;
			}
	}

	public boolean reachGlobalOptimum() {
		if (BEST_SO_FAR == UPPER_BOUND + 1)
			return true;
		else
			return false;
	}

	public void showResult(int n) {
		output.clearGridValue();
		// System.out.println("Generation : " + n);
		// System.out.println("Population : ");
		output.updatePopulation(population);
		for (int i = 0; i < population.size(); i++) {
			GridGenome ggt = (GridGenome) population.get(i);
			// System.out.println(ggt.printChromosome());
			output.updateGridValue(ggt.chromosome[0], ggt.chromosome[1]);
		}
		BEST_SO_FAR = bestGenome.getFitnessValue();
		for (int i = 0; i < nextGeneration.size(); i++) {
			GridGenome ggt = (GridGenome) nextGeneration.get(i);
			output.updateGridValue(ggt.chromosome[0], ggt.chromosome[1],
					GridFitnessLandscapeDisplay.NEXT_GENERATION_TYPE);
		}
		output.updateGridValue(bestGenome.chromosome[0],
				bestGenome.chromosome[1],
				GridFitnessLandscapeDisplay.BEST_SO_FAR_TYPE);
		output.repaint();
		// pause(100);
	}

	public void updateIndividualsOnDisplay(Genome<int[]> g1, Genome<int[]> g2,
			int step) {
		showResult(0);
		switch (step) {
		case STEP_SELECTION:
			output.updateGridValue(g1.chromosome[0], g1.chromosome[1],
					GridFitnessLandscapeDisplay.ACTIVE_TYPE);
			output.updateGridValue(g2.chromosome[0], g2.chromosome[1],
					GridFitnessLandscapeDisplay.ACTIVE_TYPE);
			break;
		case STEP_CROSSOVER:
			output.updateGridValue(g1.chromosome[0], g1.chromosome[1],
					GridFitnessLandscapeDisplay.PRODUCT_TYPE);
			output.updateGridValue(g2.chromosome[0], g2.chromosome[1],
					GridFitnessLandscapeDisplay.PRODUCT_TYPE);
			break;
		case STEP_MUTATION:
			output.updateGridValue(g1.chromosome[0], g1.chromosome[1],
					GridFitnessLandscapeDisplay.PRODUCT_TYPE);
			output.updateGridValue(g2.chromosome[0], g2.chromosome[1],
					GridFitnessLandscapeDisplay.PRODUCT_TYPE);
			break;
		default:
			output.updateGridValue(g1.chromosome[0], g1.chromosome[1],
					GridFitnessLandscapeDisplay.POPULATION_TYPE);
			output.updateGridValue(g2.chromosome[0], g2.chromosome[1],
					GridFitnessLandscapeDisplay.POPULATION_TYPE);
			break;
		}
		output.repaint();
		waitForNextStep();
	}

	public void printSummary() {
		output.excelHandler.printResults((GeneticAlgorithm<?>) this,
				ExcelHandler.GRID, PRINT_GRAPH);
	}

	public void printGraphImage() {
		output.excelHandler.printGraphImage(output, ExcelHandler.GRID);
	}

	public void updateActiveIndex(int activeIndex) {
		output.updateActiveIndex(activeIndex);
		output.repaint();
	}
}

class RealCodedGeneticAlgorithm extends GeneticAlgorithm<double[]> {

	public int testNumber = -1;

	public Expression[] expr;

	public RealCodedGeneticAlgorithm(Display output, int testNumber) {
		super(output);
		this.testNumber = testNumber;
	}

	public RealCodedGeneticAlgorithm(Display output, Expression[] expr) {
		super(output);
		this.expr = expr;
	}

	public void initDefaultParameter() {
		CHROMOSOME_SIZE = 3;
		POPULATION_SIZE = 100;
		GENERATION_SIZE = 100;
		BEST_SO_FAR = Double.MAX_VALUE;
		MUTATION_RATE = 1.0 / CHROMOSOME_SIZE;
		maximize = false;
	}

	public void initPopulation() {
		this.output.initXYGraph();
		population = PopulationFactory.get1DPopulation(new RealCodedGenome(
				chromosomeSize, this), populationSize);
		population2D = PopulationFactory.get2DPopulation(population,
				populationSizeX);
	}

	public double getDiversity(ArrayList<Genome<double[]>> a,
			ArrayList<Genome<double[]>> b) {
		double diversity = 0, temp;
		for (Genome<double[]> g1 : a) {
			for (Genome<double[]> g2 : b) {
				temp = 0;
				for (int i = 0; i < chromosomeSize; i++)
					temp += Math.abs(g1.chromosome[i] - g2.chromosome[i]);
				diversity += temp / chromosomeSize;
			}
		}
		return diversity / (a.size() * b.size());
	}
}

class MultiObjectiveRealCodedGeneticAlgorithm extends RealCodedGeneticAlgorithm {

	public int objectiveNum = 2;

	public MultiObjectiveRealCodedGeneticAlgorithm(Display output,
			int testNumber) {
		super(output, testNumber);
		initDisplay();
	}

	public MultiObjectiveRealCodedGeneticAlgorithm(Display output,
			Expression[] expr) {
		super(output, expr);
		initDisplay();
	}

	public void initDisplay() {
		this.output.initScatterGraph(MultiObjectiveTestFunction
				.getInitialLowerGraphRange(testNumber),
				MultiObjectiveTestFunction
						.getInitialUpperGraphRange(testNumber));
	}

	public void initDefaultParameter() {
		CHROMOSOME_SIZE = 3;
		POPULATION_SIZE = 100;
		GENERATION_SIZE = 1000;
		BEST_SO_FAR = Double.MAX_VALUE;
		MUTATION_RATE = 1.0 / CHROMOSOME_SIZE;
		maximize = false;
	}

	public void initPopulation() {
		population = PopulationFactory.get1DPopulation(
				new MultiObjectiveRealCodedGenome(chromosomeSize, objectiveNum,
						this), populationSize);
		population2D = PopulationFactory.get2DPopulation(population,
				populationSizeX);
		assignDominatedRank();
	}

	public void evaluate() {
		assignDominatedRank();
		super.evaluate();
	}

	public void assignDominatedRank() {
		int rank;
		for (int i = 0; i < population.size(); i++) {
			MultiObjectiveRealCodedGenome genome = (MultiObjectiveRealCodedGenome) population
					.get(i);
			genome.calculateObjectiveValue(testNumber);
		}
		for (int i = 0; i < population.size(); i++) {
			rank = 1;
			MultiObjectiveRealCodedGenome rankGenome = (MultiObjectiveRealCodedGenome) population
					.get(i);
			for (int j = 0; j < population.size(); j++) {
				MultiObjectiveRealCodedGenome compareGenome = (MultiObjectiveRealCodedGenome) population
						.get(j);
				if (compareGenome.dominate(rankGenome, maximize))
					rank++;
			}
			rankGenome.setRank(rank);
		}
	}

	public int getRank(int index) {
		MultiObjectiveRealCodedGenome genome = (MultiObjectiveRealCodedGenome) population
				.get(index);
		return genome.getRank();
	}

	public boolean isBetter(Genome<double[]> g1, Genome<double[]> g2) {
		((MultiObjectiveRealCodedGenome) g1)
				.calculateObjectiveValue(testNumber);
		((MultiObjectiveRealCodedGenome) g2)
				.calculateObjectiveValue(testNumber);
		return ((MultiObjectiveRealCodedGenome) g1).dominate(
				((MultiObjectiveRealCodedGenome) g2), maximize);
	}

	public void showResult(int n) {
		output.clearGraphValue(0);
		for (int i = 0; i < population.size(); i++) {
			MultiObjectiveRealCodedGenome genome = (MultiObjectiveRealCodedGenome) population
					.get(i);
			output.updateGraphValue(0, 0, genome.objectiveValue[0],
					genome.objectiveValue[1]);
		}
		output.updateAlgorithmStatus(n, evaluationCount);
		output.updateGraphImage();
	}
}

class SingleObjectiveTestFunction {

	public static final String[] FUNCTION_LIST_STRING = { "x*x",
			"-1/(x*x*5+y*y*5+0.7)" };

	// VARIABLE_RANGE[testNumber][lowerUpper]
	public static double[][] VARIABLE_DOMAIN = { { -1000.0, 1000.0 },
			{ 0.0, 1.0 } };

	public static double[] USER_DEFINED_VARIABLE_DOMAIN = { 0.0, 1.0 };

	public static double calculateFunctionValue(int index, double[] x) {
		switch (index) {
		case 0:
			return x[0] * x[0];
		case 1:
			return -1 / (x[0] * x[0] * 5 + x[1] * x[1] * 5 + 0.7);
		default:
			return calculateFunctionValue(0, x);
		}
	}

	public static double getLowerDomain(int testNumber) {
		if (GeneticAlgorithm.USE_PREDEFINED_FUNCTION)
			return VARIABLE_DOMAIN[testNumber][0];
		else
			return USER_DEFINED_VARIABLE_DOMAIN[0];
	}

	public static double getUpperDomain(int testNumber) {
		if (GeneticAlgorithm.USE_PREDEFINED_FUNCTION)
			return VARIABLE_DOMAIN[testNumber][1];
		else
			return USER_DEFINED_VARIABLE_DOMAIN[1];
	}
}

class MultiObjectiveTestFunction {

	public static final int[][] FUNCTION_LIST = { { 0, 1 }, { 2, 5 }, { 2, 7 } };

	public static String[][] FUNCTION_LIST_STRING = { { "x*x", "(x-2)*(x-2)" },
			{ "x", "(1+9*(y+z))*(1-sqrt(x/(1+9*(y+z))))" },
			{ "x", "1-pow(x,0.25)-x*sin(10*PI*x)" } };

	// VARIABLE_RANGE[testNumber][lowerUpper]
	public static double[][] VARIABLE_DOMAIN = { { -1000.0, 1000.0 },
			{ 0.0, 1.0 }, { 0.0, 1.0 } };

	public static double[] USER_DEFINED_VARIABLE_DOMAIN = { 0.0, 1.0 };

	public static double calculateFunctionValue(int index, double[] x) {
		double temp;
		switch (index) {
		case 0:
			return x[0] * x[0];
		case 1:
			return (x[0] - 2) * (x[0] - 2);
		case 2: // f1
			return x[0];
		case 3: // g
			double sum = 0;
			for (int i = 1; i < x.length; i++) {
				sum += x[i];
			}
			return 1 + 9 * sum;
		case 4: // h1
			temp = Math.sqrt(calculateFunctionValue(2, x)
					/ calculateFunctionValue(3, x));
			return 1 - temp;
		case 5: // f12
			return calculateFunctionValue(3, x) * calculateFunctionValue(4, x);
		case 6: // h2
			temp = Math.pow(calculateFunctionValue(2, x)
					/ calculateFunctionValue(3, x), 2);
			return 1 - temp;
		case 7: // f22
			return calculateFunctionValue(3, x) * calculateFunctionValue(6, x);
		default:
			return calculateFunctionValue(2, x);
		}
	}

	public static double[] getInitialLowerGraphRange(int testNumber) {
		double[] result = { 0.0, 0.0 };
		switch (testNumber) {
		case 0:
			result[0] = 0.0;
			result[1] = 0.0;
			return result;
		case 1:
			result[0] = 0.0;
			result[1] = 0.0;
			return result;
		case 2:
			result[0] = 0.0;
			result[1] = 0.0;
			return result;
		default:
			return result;
		}
	}

	public static double[] getInitialUpperGraphRange(int testNumber) {
		double[] result = { 1.0, 20.0 };
		switch (testNumber) {
		case 0:
			result[0] = 100000.0;
			result[1] = 100000.0;
			return result;
		case 1:
			result[0] = 1.0;
			result[1] = 20.0;
			return result;
		case 2:
			result[0] = 1.0;
			result[1] = 20.0;
			return result;
		default:
			return result;
		}
	}

	public static double getLowerDomain(int testNumber) {
		if (GeneticAlgorithm.USE_PREDEFINED_FUNCTION)
			return VARIABLE_DOMAIN[testNumber][0];
		else
			return USER_DEFINED_VARIABLE_DOMAIN[0];
	}

	public static double getUpperDomain(int testNumber) {
		if (GeneticAlgorithm.USE_PREDEFINED_FUNCTION)
			return VARIABLE_DOMAIN[testNumber][1];
		else
			return USER_DEFINED_VARIABLE_DOMAIN[1];
	}
}
